/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package javax.microedition.location;

/**
 * Geographic coordinates.<p>
 * 
 * The <code>Coordinates</code> class represents coordinates as latitude-longitude-altitude values. 
 * The latitude and longitude values are expressed in degrees using floating point values. 
 * The degrees are in decimal values (rather than minutes/seconds). 
 * The coordinates are given using the WGS84 datum.<p>
 * 
 * This class also provides convenience methods for converting between a <code>String</code> coordinate representation and the <code>double</code> representation used in this class. 
 * 
 * @author faerber
 * @author abiskop
 * @author king
 */
public class Coordinates {

	private float altitude;
	private double latitude, longitude;

	public static final int DD_MM_SS = 1;
	public static final int DD_MM = 2;
	
	private static final double DEFAULT_PRECISION = 0.00000000005;
	private static final int MAX_LOOP_EXECUTIONS = 1000;
	private static final double DELTA = 0.0000005;

	/**
	 * Constructs a new <code>Coordinates</code> object with the values specified.
	 * The latitude and longitude parameters are expressed in degrees using floating point values.
	 * The degrees are in decimal values (rather than minutes/seconds).
	 * <p>The coordinate values always apply to the WGS84 datum.</p>
	 * <p>The Float.NaN value can be used for altitude to indicate that altitude is not known.</p>
	 *  
	 * @param latitude	the latitude of the location. Valid range: [-90.0, 90.0]. Positive values indicate northern latitude and negative values southern latitude.
	 * @param longitude	the longitude of the location. Valid range: [-180.0, 180.0). Positive values indicate eastern longitude and negative values western longitude.
	 * @param altitude	the altitude of the location in meters, defined as height above the WGS84 ellipsoid. <code>Float.NaN</code> can be used to indicate that altitude is not known.
	 * 
	 * @throws IllegalArgumentException	if an input parameter is out of the valid range
	 */
	public Coordinates(double latitude, double longitude, float altitude) {
		if ((latitude < -90.0) || (latitude > 90.0))
			throw new IllegalArgumentException("Invalid latitude.");
		if ((longitude < -180.0) || (longitude > 180.0))
			throw new IllegalArgumentException("Invalid longitude.");
			
		this.latitude = latitude;
		this.longitude = longitude;
		this.altitude = altitude;
	}

	/**
	 * Gets the altitude.
	 * 
	 * @return	altitude
	 */
	public float getAltitude() {
		return altitude;
	}

	/**
	 * Gets the latitude.
	 * 
	 * @return	latitude
	 */
	public double getLatitude() {
		return latitude;
	}

	/**
	 * Gets the longitude.
	 * 
	 * @return	longitude
	 */
	public double getLongitude() {
		return longitude;
	}

	/**
	 * Sets the altitude.
	 * 
	 * @param altitude
	 */
	public void setAltitude(float altitude) {
		this.altitude = altitude;
	}

	/**
	 * Sets the latitude.
	 * 
	 * @param latitude
	 */
	public void setLatitude(double latitude) {
		this.latitude = latitude;
	}

	/**
	 * Sets the longitude.
	 * 
	 * @param longitude
	 */
	public void setLongitude(double longitude) {
		this.longitude = longitude;
	}
	
	/**
	 * Calculates the azimuth between the two points according to the ellipsoid model of WGS84. <br />
	 * The azimuth is relative to true north.<br />
	 * The Coordinates object on which this method is called is considered the origin for the calculation
	 * and the Coordinates object passed as a parameter is the destination which the azimuth is calculated to.
	 * When the origin is the North pole and the destination is not the North pole, this method returns 180.0.
	 * When the origin is the South pole and the destination is not the South pole, this method returns 0.0.
	 * If the origin is equal to the destination, this method returns 0.0.<br />
	 * The implementation shall calculate the result as exactly as it can.
	 * However, it is required that the result is within 1 degree of the correct result.<p>
	 * 
	 * <u>Important notice:</u><br />
	 * The azimuth of two coordinates can be calculated in two different ways:<p>
	 * 
	 * 1) Astronomical azimuth.<br />
	 * This defines the azimuth as follows:<br />
	 * "This is the direction of a celestial object, measured clockwise around the observer's horizon from north.
	 * So an object due north has an azimuth of 0°, one due east 90°, south 180° and west 270°.
	 * Azimuth and altitude are usually used together to give the direction of an object in the topocentric coordinate system.<br />
	 * (...)<br />
	 * The azimuth of a point on the celestial sphere is defined as the angular distance
	 * measured towards the east, from north, along the astronomical horizon to the intersection
	 * of the great circle passing through the point and the astronomical zenith with the astronomical horizon."<br />
	 * (Taken from: http://www.heavens-above.com)<br />
	 * This means that for example the azimuth from the origin (50:00.00, 50:00.00) towards the destination (70:00.00, 70:00.00) would be exactly 45° ("North-East", "NE").<p>
	 * 
	 * 2) Ellipsoidal azimuth.<br />
	 * The ellipsoidal azimuth differs from the astronomical azimuth by the east-west-component of the"perpendicular deviation" (dt.: Ost-West-Komponente der Lotabweichung).<br />
	 * It takes into account that earth is not a perfect sphere but flattened at the poles and therefore uses the reference ellipsoid (e.g. WGS84) as a base for calculation.<br />
	 * Furthermore, it regards the fact that there are always two possible ways to reach a destination on earth when e.g. travelling by aeroplane, and that usually one of the two routes is shorter than the other one.
	 * For example, when travelling from New York to Beijing by plane, it would be shorter to fly across the Pacific ocean instead of crossing the Atlantic ocean and Europe/Northern Africa.
	 * As a result, the ellipsoidal azimuth with New York as the origin and Beijing as the destination would "point" towards the west.<p>
	 * 
	 * As a sloppy definition, it can be said that the ellipsoidal azimuth points into the optimal direction to fly in when wanting to reach a destination.<p>
	 * 
	 * This method, respectively the algorithm it uses, calculates the <u>ellipsoidal azimuth</u> (definition number 2).<p>
	 * 
	 * <u>Additional information:</u><br />
	 * The algorithm can handle critical coordinates (north/south pole, coordinates on the same meridian, etc.) and
	 * it can determine which direction of the two that are possible is better.<br />
	 * The precision of the result depends on the distance between origin and destination.
	 * For "short" distances (less than 10000 km) the maximum deviation is about 1° compared to the values that are calculated 
	 * by the reference implementation by Nokia.<br />
	 * But the mistake seems to be exponentially growing with the distance, as it "explodes" as the distance increases beyond 10000km.<br />
	 * Due to lack of reliable information, we do not know where this deviation between the two implementation comes from.<p>
	 * 
	 * One example where the difference between the results becomes obvious is the azimuth from Beijing, China, to Sao Paulo, Brazil.<br />
	 * Nokia's implementation calculates an azimuth of about 316 degrees.<br />
	 * Our implementation gives about 235 degrees as a result.<br />
	 * Looking at a map, we think that 235° would be the more reasonable result compared to 316°, as 316° would "point" towards the north-west, 
	 * which is not really the direction that Sao Paulo lies in, Beijing as the origin. Travelling strictly into that direction, after some time one would end up at the north pole.<br />
	 * As we do not know which additional information Nokia uses in its algorithm, we can not clearly say which of the two implementations is correct.<p>
	 * 
	 * Considering the fact that it would be a rare exception that such long distances occur in our applications 
	 * and that therefore the mistake is always rather small, we simply neglect the (potential) mistake in our azimuth calculations. 
	 * 
	 * 
	 * @param to  the <code>Coordinates</code> of the destination
	 * 
	 * @return the azimuth to the destination in degrees. Result is within the range [0.0 ,360.0).
	 * 
	 * @throws NullPointerException  if the parameter is <code>null</code>
	 * 
	 * @see <a href=http://de.wikipedia.org/wiki/Azimut>http://de.wikipedia.org/wiki/Azimut</a>
	 * @see <a href=http://en.wikipedia.org/wiki/Azimuth>http://en.wikipedia.org/wiki/Azimuth</a>
	 * @see <a href=http://www.heavens-above.com/gloss.asp?term=azimuth>http://www.heavens-above.com/gloss.asp?term=azimuth</a>
	 */
	public float azimuthTo(Coordinates to) {
		if (to == null) {
			throw new NullPointerException("Destination coordinates not specified.");
		}
		if ((latitude == to.getLatitude()) && (longitude == to.getLongitude())) {
			// --> if origin and destination are the same, the method shall return 0.0°
			return 0.0f;
		}
		if (latitude == 90.0) {
			// --> if the origin is the north pole, but the destination is not the north pole, the method shall return 180.0° 
			return 180.0f;
		}
		if (latitude == -90.0) {
			// --> if the origin is the south pole, it does not matter, where the destination is; the method will always return 0.0°
			return 0.0f;
		}
		
		double latOrig = latitude;
		double lonOrig = longitude;
		double latDest = to.getLatitude();
		double lonDest = to.getLongitude();
		
		// Determine which coordinate is on which side of the prime meridian
		if (((lonOrig < 0) && (lonDest > 0)) || ((lonOrig > 0) && (lonDest < 0))) {
			double eastLon;
			double westLon;
			if (lonDest > lonOrig) {
				eastLon = lonDest;
				westLon = lonOrig;
			} else {
				eastLon = lonOrig;
				westLon = lonDest;
			}
			/* Check which direction is shorter.
			 * Note that only the longitude is relevant here.
			 */
			double directDistance = Math.abs(eastLon - westLon);
			double indirectDistance = Math.abs(180.0 - eastLon) + Math.abs(-180.0 - westLon);
			if (indirectDistance < directDistance) {
				lonOrig = shiftLongitudeBy180Degrees(lonOrig);
				lonDest = shiftLongitudeBy180Degrees(lonDest);
			}
		}
		
		// Coordinates that are located on the same parallel
		if (latOrig == latDest) {
			if (lonOrig < lonDest) {
				return 90.0f;
			} else if (lonOrig > lonDest) {
				return 270.0f;
			}
		}
		// Coordinates that are located on the same meridian
		if (lonOrig == lonDest) {
			if (latOrig < latDest) {
				return 0.0f;
			} else if (latOrig > latDest) {
				return 180.0f;
			}
		}
		
		// Helper coordinates
		double lat0 = latDest;
		double lon0 = lonOrig;
		
		// Calculate distances
		double a = calculateVincentyDistanceWgs84(lat0, latDest, lon0, lonDest);
		double b = calculateVincentyDistanceWgs84(latOrig, lat0, lonOrig, lon0);
		double c = calculateVincentyDistanceWgs84(latOrig, latDest, lonOrig, lonDest);
		
		// Angle between the line that connects origin and destination and the line that connects origin and helper coordinate
		double phi = Math.toDegrees(Math.acos((Math.pow(a, 2) - Math.pow(b, 2) - Math.pow(c, 2)) / (-2 * b * c)));
		
		// Depending on where the destination is located relative to the origin, alpha (azimuth) is calculated from phi
		double alpha = Double.NaN;
		if (latDest > latOrig) {
			if (lonDest > lonOrig) {
				alpha = phi;
			} else if (lonDest < lonOrig) {
				alpha = 360.0 - phi;
			}
		}
		else if (latDest < latOrig) {
			if (lonDest > lonOrig) {
				alpha = 180.0 - phi;
			} else if (lonDest < lonOrig) {
				alpha = 180.0 + phi;
			}
		}
		
		return (float) alpha;

		//throw new Exception("Not implemented");
		//return Float.NaN;
	}
	
	private double shiftLongitudeBy180Degrees(double lon) {
		if (lon > 0.0) {
			return lon - 180.0;
		} else if (lon < 0.0) {
			return lon + 180.0;
		} else {
			return Double.NaN;
		}
	}

	/**
	 * Calculates the geodetic distance between the two points according to the ellipsoid model of WGS84.
	 * Altitude is neglected from calculations.
	 * The algorithm by T. Vincenty is used.
	 * 
	 * @param to  the <code>Coordinates</code> of the destination
	 * 
	 * @return the distance to the destination in meters
	 * 
	 * @throws NullPointerException  if the parameter is <code>null</code>
	 * 
	 * @see <a href=http://de.wikipedia.org/wiki/Referenzellipsoid>http://de.wikipedia.org/wiki/Referenzellipsoid</a>
	 */
	public float distance(Coordinates to) {
		if ((latitude == to.getLatitude()) && (longitude == to.getLongitude())) {
			return 0.0f;
		} else {
			return (float) calculateVincentyDistanceWgs84(
				latitude,
				to.getLatitude(),
				longitude,
				to.getLongitude());
		}
	}
	
	/**
	 * Converts a String representation of a coordinate into the double
	 * representation as used in this API.<p>
	 * 
	 * There are two string syntaxes supported:<p>
	 * 
	 * 1. Degrees, minutes, seconds and decimal fractions of seconds.
	 * This is expressed as a string complying with the following BNF definition where the degrees are
	 * within the range [-179, 179] and the minutes and seconds are within the range [0, 59],
	 * or the degrees is -180 and the minutes, seconds and decimal fractions are 0:<p>
	 * <pre>
	 * coordinate = degrees ":" minutes ":" seconds "." decimalfrac
	 * 						| degrees ":" minutes ":" seconds
	 * 						| degrees ":" minutes
	 * degrees = degreedigits | "-" degreedigits
	 * degreedigits = digit | nonzerodigit digit | "1" digit digit
	 * minutes = minsecfirstdigit digit
	 * seconds = minsecfirstdigit digit
	 * decimalfrac = 1*3digit
	 * digit = "0" | "1" | "2" | "3" | "4" | "5" | "6" | "7" | "8" | "9"
	 * nonzerodigit = "1" | "2" | "3" | "4" | "5" | "6" | "7" | "8" | "9"
	 * minsecfirstdigit = "0" | "1" | "2" | "3" | "4" | "5"
	 * </pre>
	 * <p>
	 * 
	 * 2. Degrees, minutes and decimal fractions of minutes.
	 * This is expressed as a string complying with the
	 * following BNF definition where the degrees are within the
	 * range [-179, 179] and the minutes are within the range [0, 59], or the degrees is -180 and the
	 * minutes and decimal fractions are 0:<p>
	 * <pre>
	 * coordinate = degrees ":" minutes "." decimalfrac
	 * 						| degrees ":" minutes
	 * degrees = degreedigits | "-" degreedigits
	 * degreedigits = digit | nonzerodigit digit | "1" digit digit
	 * minutes = minsecfirstdigit digit
	 * decimalfrac = 1*5digit
	 * digit = "0" | "1" | "2" | "3" | "4" | "5" | "6" | "7" | "8" | "9"
	 * nonzerodigit = "1" | "2" | "3" | "4" | "5" | "6" | "7" | "8" | "9"
	 * minsecfirstdigit = "0" | "1" | "2" | "3" | "4" | "5"
	 * </pre>
	 * <p>
	 * For example, for the <code>double</code> value of the coordinate <code>61.51d</code>, the corresponding syntax 1
	 * string is <code>"61:30:36"</code> and the corresponding syntax 2 string is <code>"61:30.6"</code>. 
	 * 
	 * @param coordinate  a String in either of the two representation specified above
	 * 
	 * @return a double value with decimal degrees that matches the string representation given as the parameter
	 * 
	 * @throws IllegalArgumentException  if the coordinate input parameter does not comply with the defined syntax for the specified types
	 */
	public static double convert(String coordinate) {
		try {
			int degrees = Integer.parseInt(coordinate.substring(0, coordinate.indexOf(':')));
			double minutes = Double.parseDouble(coordinate.substring(coordinate.indexOf(':') + 1));
			if (degrees < 0) {
				return (double) degrees - (minutes / 60.0);
			} else {
				return (double) degrees + (minutes / 60.0);
			}
		} catch (Exception e) {
			throw new IllegalArgumentException("Illegal coordinate string");
		}
	}
	
	/**
	 * Converts a double representation of a coordinate with decimal degrees 
	 * into a string representation.<p>
	 * 
	 * There are string syntaxes supported are the same as for the <code>convert(String)</code> method. 
	 * The implementation shall provide as many significant digits for the decimal fractions as 
	 * are allowed by the string syntax definition.
	 * 
	 * @param coordinate  a double reprentation of a coordinate
	 * @param outputType  identifier of the type of the string representation wanted for output The constant DD_MM_SS identifies the syntax 1 and the constant DD_MM identifies the syntax 2.
	 * 
	 * @return a string representation of the coordinate in a representation indicated by the parameter
	 * 
	 * @throws IllegalArgumentException  if the outputType is not one of the two constant values defined in this class or if the coordinate value is not within the range [-180.0, 180.0) or is Double.NaN
	 * 
	 * @see #convert(String coordinate)
	 */
	public static String convert(double coordinate, int outputType) {
		if ((coordinate < -180) || (coordinate > 180) || Double.isNaN(coordinate)) {
			throw new IllegalArgumentException("Coordinate value out of range or not a number");
		}
		try {			
			int degrees;
			double decfrac;
			if (coordinate >= 0) {
				degrees = (int) Math.floor(coordinate);
				decfrac = coordinate - degrees;				
			} else {
				degrees = (int) Math.ceil(coordinate);
				decfrac = -1 * (coordinate - degrees);
			}
			
			int minutes = (int) Math.floor(decfrac * 60.0d);
			int minfrac = (int) Math.rint(((decfrac * 60.0d) - minutes) * 100.0d); // two decimal digits
			
			StringBuffer output = new StringBuffer();
			switch (outputType) {
				case DD_MM : {
					int sign = 1;
					if ((degrees < 10.0) && (degrees >= 0.0))
						output.append(0);
					if ((degrees < 0.0) && (degrees > -10.0)) {
						output.append(-0);
						sign = -1;
					}
					output.append(sign * degrees);
					output.append(':');
					if (minutes < 10)
						output.append(0);
					output.append(minutes);
				}
				break;
				case DD_MM_SS : {
					int sign = 1;
					if ((degrees < 10.0) && (degrees >= 0.0))
						output.append(0);
					if ((degrees < 0.0) && (degrees > -10.0)) {
						output.append(-0);
						sign = -1;
					}
					output.append(sign * degrees);
					output.append(':');
					if (minutes < 10)
						output.append(0);
					output.append(minutes);
					output.append('.');
					if (minfrac < 10)
						output.append(0);
					output.append(minfrac);
				}
				break;
				default :
					throw new IllegalArgumentException("Illegal output type");
			}
			return output.toString();
		} catch (Exception e) {
			throw new IllegalArgumentException("Illegal coordinate or output type");
		}
	}

	private static double calculateVincentyDistanceWgs84(
		double latitude1, double latitude2, double longitude1, double longitude2) {
			
		return calculateVincentyDistanceWgs84(
			latitude1,
			latitude2,
			longitude1,
			longitude2,
			DEFAULT_PRECISION);
	}

	private static double calculateVincentyDistanceWgs84(double latitude1, double latitude2, double longitude1,
		double longitude2,	double precision) {
		
		// Check for critical coordinates
		// -----------------------------------------
		// If lat1 and lat2 are the same, the algorithm cannot calculate the distance.
		// Therefore, a small value (DELTA) is added resp. subtracted from lat2 so that calculation can take place.
		// If unfortunately lat2 is an edge value (>= 89.0 or <= -89.0), this could cause problems, 
		// wherefore this is checked and then DELTA is added or subtracted, depending on what value lat2 has.
		// The same is done for longitude afterwards.
		double tempLat2 = latitude2;
		if (latitude1 == latitude2) {
			if (latitude2 >= 89.0) {
				tempLat2 -= DELTA;
			} else {
				tempLat2 += DELTA;
			}
		}
		double tempLon2 = longitude2;
		if (longitude1 == longitude2) {
			if (longitude2 >= 179.0) {
				tempLon2 -= DELTA;
			} else {
				tempLon2 += DELTA;
			}
		}
		
		// conversion from degrees to radians		
		final double LAT1 = Math.toRadians(latitude1);
		final double LAT2 = Math.toRadians(tempLat2);
		final double LON1 = Math.toRadians(longitude1);
		final double LON2 = Math.toRadians(tempLon2);

		/* 
		 * WGS84 (World Geodetic System 1984) axis definitions
		 * 
		 * Axis definitions taken from:
		 * http://de.wikipedia.org/wiki/Referenzellipsoid
		 * 
		 * a0 = 6378137.000 ("long axis")
		 * b0 = 6356752.315 ("short axis")
		 * 
		 * These values are used to calculate the "flatness" of the coordinate system.
		 * Additionally, they are needed at the end of the algorithm to do some computations.
		 */
		final float A0 = 6378137.000f;
		final float B0 = 6356752.315f;
		
		final double FLAT = (A0 - B0) / A0;

		double u1 = Math.atan((1 - FLAT) * Math.tan(LAT1));
		double u2 = Math.atan((1 - FLAT) * Math.tan(LAT2));
		
		/* Check which direction is shorter.
		 * According to this, omega is initially set to either "|lon2 - lon1|" or "360° - |lon2 - lon1|" resp. "2 * PI - |lon2 - lon1|".
		 */
		double omega = Math.abs(LON2 - LON1);
		if (omega > Math.PI) {
			omega = 2 * Math.PI - omega;
		}
		
		/* 
		 * lambda is gradually improved during the algorithm.
		 * initially, it is set to omega.
		 */
		double lambda = omega;
		 
		/* 
		 * For these variables, there is no reasonable "starting value".
		 * Their values are computed step by step by the algorithm.
		 * Therefore they are initially set to Double.NaN. 
		 */		 
		double lambdaOld = Double.NaN;
		double sinSigma = Double.NaN;
		double cosSigma = Double.NaN;
		double sigma = Double.NaN;
		double alpha = Double.NaN;
		double cos2sigmam = Double.NaN;
		double c = Double.NaN;
		double a = Double.NaN;
		double b = Double.NaN;
		double s = Double.NaN;
		double deltasigma = Double.NaN;

		// now the real algorithm starts
		boolean executeLoop = true;
		int i = 0; // iterator that counts the loop executions
		
		while ((Math.abs(lambdaOld - lambda) > precision) || executeLoop) {
			// force at least one execution so that the variables have a reasonable value
			executeLoop = false;
			
			if(i >= MAX_LOOP_EXECUTIONS) { // make sure that the loop is not executed too often
				break;
			}
			i++;

			lambdaOld = lambda;
			
			/* 
			 * The following statement computes this term:
			 * sinSigma = sqrt((cos(U2) * sin(lambda))^2 + (cos(U1) * sin(U2) - sin(U1) * cos(U2) * cos(lambda))^2);
			 */
			sinSigma = Math.sqrt(Math.pow(Math.cos(u2) * Math.sin(lambda), 2)
								+ Math.pow(Math.cos(u1) * Math.sin(u2)
								 - Math.sin(u1) * Math.cos(u2) * Math.cos(lambda), 2));

			cosSigma =	Math.sin(u1) * Math.sin(u2) + Math.cos(u1) * Math.cos(u2) * Math.cos(lambda);

			sigma = Math.atan2(sinSigma, cosSigma);

			alpha = Math.asin(Math.cos(u1)	* Math.cos(u2) * Math.sin(lambda)	/ Math.sin(sigma));

			cos2sigmam = Math.cos(sigma)	- 2 * Math.sin(u1) * Math.sin(u2)	/ Math.pow(Math.cos(alpha), 2);
			
			/* 
			 * The following statement computes this term:
			 * c = (FLAT / 16) * cos(alpha)^2 * (4 + FLAT * (4 - 3 * cos(alpha)^2));
			 */
			c = (FLAT / 16.0) * Math.pow(Math.cos(alpha), 2) * (4.0 + FLAT * (4.0 - 3.0 * Math.pow(Math.cos(alpha), 2)));
			
			/* 
			 * The following statement computes this term:
			 * lambda = omega + (1 - c) * FLAT * sin(alpha) * (sigma + c * sin(sigma) * (cos2sigmam + c * cos(sigma) * (-1 + 2 * cos2sigmam^2)))
			 */
			lambda = omega	+ (1 - c) * FLAT * Math.sin(alpha)
						* (sigma + c * Math.sin(sigma)
							* (cos2sigmam + c * Math.cos(sigma)
						 		* (- 1.0 + 2.0 * Math.pow(cos2sigmam, 2))));
		}
		/* 
		 * The following statement computes this term:
		 * u2 = cos(alpha)^2 * (a^2 - b^2) / b^2;
		 */
		u2 = Math.pow(Math.cos(alpha), 2) * (Math.pow(A0, 2) - Math.pow(B0, 2)) / Math.pow(B0, 2);

		a = 1 + u2 / 16384.0 * (4096.0 + u2 * (-768.0 + u2 * (320.0 - 175.0 * u2)));
		b = (u2 / 1024.0) * (256.0 + u2 * (-128.0 + u2 * (74.0 - 47.0 * u2)));

		deltasigma = b * Math.sin(sigma) * (cos2sigmam + (b / 4)	
							 * (Math.cos(sigma) * (-1.0 + 2.0 * Math.pow(cos2sigmam, 2))
							 - (b / 6) * cos2sigmam * (-3.0 + 4.0 * Math.pow(Math.sin(sigma), 2))
							 * (-3.0 + 4.0 * Math.pow(cos2sigmam, 2))));

		s = B0 * a * (sigma - deltasigma);
		return s;
	}
}
