/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package javax.microedition.location;

/**
 * The criteria used for the selection of the location provider is defined by the values in this class.
 * It is up to the implementation to provide a <code>LocationProvider</code> that can obtain
 * locations constrained by these values.
 * <p>Instances of <code>Criteria</code> are used by the application to indicate criteria
 * for choosing the location provider in the <code>LocationProvider.getInstance</code> method call.
 * The implementation considers the different criteria fields to choose the location provider
 * that best fits the defined criteria. The different criteria fields do not have any defined priority order
 * but the implementation uses some implementation specific logic to choose the location provider
 * that can typically best meet the defined criteria.</p>
 * <p>However, the cost criteria field is treated differently from others. If the application has set
 * the cost field to indicate that the returned location provider is not allowed to incur financial cost
 * to the end user, the implementation MUST guarantee that the returned location provider
 * does not incur cost.</p>
 * <p>If there is no available location provider that is able to meet all the specified criteria,
 * the implementation is allowed to make its own best effort selection of a location provider
 * that is closest to the defined criteria (provided that the cost criteria is met).
 * However, an implementation is not required to return a location provider if it does not have any
 * available provider that is able to meet these criteria or be sufficiently close to meeting them,
 * where the judgement of sufficiently close is an implementation dependent best effort choice.
 * It is left up to the implementation to consider what is close enough to the specified requirements
 * that it is worth providing the location provider to the application.</p>
 * <p>The default values for the criteria fields are specified below in the table.
 * The default values are always the least restrictive option that will match all location providers.
 * Default values:</p>
 * <table>
 * <thead>
 * <tr><th>Criteria field</th><th>Default value</th></tr>
 * </thead>
 * <tbody>
 * <tr><td>Horizontal accuracy</td><td>NO_REQUIREMENT</td></tr>
 * <tr><td>Vertical accuracy</td><td>NO_REQUIREMENT</td></tr>
 * <tr><td>Preferred response time</td><td>NO_REQUIREMENT</td></tr>
 * <tr><td>Power consumption</td><td>NO_REQUIREMENT</td></tr>
 * <tr><td>Cost allowed</td><td>true (allowed to cost)</td></tr>
 * <tr><td>Speed and course required</td><td>false (not required)</td></tr>
 * <tr><td>Altitude required</td><td>false (not required)</td></tr>
 * <tr><td>Address info required</td><td>false (not required)</td></tr>
 * </tbody>
 * </table>
 * <p>The implementation of this class only retains the values that are passed in
 * using the set* methods. It does not try to validate the values of the parameters in any way.
 * Applications may set any values it likes, even negative values, but the consequence may be
 * that no matching <code>LocationProvider</code> can be created.</p>
 * 
 * @author faerber
 */
public class Criteria {
	
	private int powerPref, vertAccuracy, horzAccuracy, responseTimePref;
	private boolean costAllowed, altitudeRequired, speedAndCourseRequired, addressInfoRequired;
	
	/**
	 * Constant indicating no requirements for the parameter.
	 */
	public static final int NO_REQUIREMENT = 0;
	
	/**
	 * Level indicating only low power consumption allowed.
	 */
	public static final int POWER_USAGE_LOW = 1;
	
	/**
	 * Level indicating average power consumption allowed.
	 */
	public static final int POWER_USAGE_MEDIUM = 2;
	
	/**
	 * Level indicating high power consumption allowed.
	 */
	public static final int POWER_USAGE_HIGH = 3;
	
	
	/**
	 * Constructs a <code>Criteria</code> object.
	 * All the fields are set to the default values that are specified below
	 * in the specification of the set* methods for the parameters.
	 */
	public Criteria() {
		horzAccuracy = NO_REQUIREMENT;
		vertAccuracy = NO_REQUIREMENT;
		responseTimePref = NO_REQUIREMENT;
		powerPref = NO_REQUIREMENT;
		costAllowed = true;
		speedAndCourseRequired = false;
		altitudeRequired = false;
		addressInfoRequired = false;
	}
	
	//////////////////////////
	// getters
	
	/**
	 * Returns the preferred power consumption.
	 * 
	 * @return	the power consumption level, should be one of <code>NO_REQUIREMENT</code>, <code>POWER_USAGE_LOW</code>, <code>POWER_USAGE_MEDIUM</code>, <code>POWER_USAGE_HIGH</code>.
	 */
	public int getPreferredPowerConsumption() {
		return powerPref;
	}
	
	/**
	 * Returns the preferred cost setting.
	 * 
	 * @return	the preferred cost setting. <code>true</code> if allowed to cost, <code>false</code> if it must be free of charge.
	 */
	public boolean isAllowedToCost() {
		return costAllowed;
	}
	
	/**
	 * Returns the vertical accuracy value set in this <code>Criteria</code>.
	 * 
	 * @return	the accuracy in meters
	 */
	public int getVerticalAccuracy() {
		return vertAccuracy;
	}
	
	/**
	 * Returns the horizontal accuracy value set in this <code>Criteria</code>.
	 * 
	 * @return	the horizontal accuracy in meters
	 */
	public int getHorizontalAccuracy() {
		return horzAccuracy;
	}
	
	/**
	 * Returns the preferred maximum response time.
	 * 
	 * @return	the maximum response time in milliseconds
	 */
	public int getPreferredResponseTime() {
		return responseTimePref;
	}
	
	/**
	 * Returns whether the location provider should be able to determine speed and course.
	 * 
	 * @return	whether the location provider should be able to determine speed and course. <code>true</code> means that it should be able, <code>false</code> means that this is not required.
	 */
	public boolean isSpeedAndCourseRequired() {
		return speedAndCourseRequired;
	}
	
	/**
	 * Returns whether the location provider should be able to determine altitude.
	 * 
	 * @return	whether the location provider should be able to determine altitude. <code>true</code> means that it should be able, <code>false</code> means that this is not required.
	 */
	public boolean isAltitudeRequired() {
		return altitudeRequired;
	}
	
	/**
	 * Returns whether the location provider should be able to determine textual address information.
	 * 
	 * @return	whether the location provider should be able to normally provide textual address information. <code>true</code> means that it should be able, <code>false</code> means that this is not required.
	 */
	public boolean isAddressInfoRequired() {
		return addressInfoRequired;
	}
	
	/////////////////////////////////////////////////////////////////////////////
	// setters
	
	/**
	 * Sets the desired horizontal accuracy preference. Accuracy is measured in meters.
	 * The preference indicates maximum allowed typical 1-sigma standard deviation
	 * for the location method.
	 * Default is <code>NO_REQUIREMENT</code>, meaning no preference on horizontal accuracy.
	 * 
	 * @param accuracy	the preferred horizontal accuracy in meters
	 */
	public void setHorizontalAccuracy(int accuracy) {
		horzAccuracy = accuracy;
	}
	
	/**
	 * Sets the desired vertical accuracy preference. Accuracy is measured in meters.
	 * The preference indicates maximum allowed typical 1-sigma standard deviation
	 * for the location method.
	 * Default is <code>NO_REQUIREMENT</code>, meaning no preference on vertical accuracy.
	 * 
	 * @param accuracy	the preferred vertical accuracy in meters
	 */
	public void setVerticalAccuracy(int accuracy) {
		vertAccuracy = accuracy;
	}
	
	/**
	 * Sets the desired maximum response time preference.
	 * This value is typically used by the implementation to determine a location method
	 * that typically is able to produce the location information within the defined time.
	 * Default is <code>NO_REQUIREMENT</code>, meaning no response time constraint.
	 * 
	 * @param time	the preferred time constraint and timeout value in milliseconds
	 */
	public void setPreferredResponseTime(int time) {
		responseTimePref = time;
	}
	
	/**
	 * Sets the preferred maximum level of power consumption.
	 * <p>These levels are inherently indeterminable and depend on many factors.
	 * It is the judgement of the implementation that defines a positioning method as consuming
	 * low power or high power.
	 * Default is <code>NO_REQUIREMENT</code>, meaning power consumption
	 * is not a quality parameter.</p>
	 *  
	 * @param level	the preferred maximum level of power consumption. Should be one of <code>NO_REQUIREMENT</code>, <code>POWER_USAGE_LOW</code>, <code>POWER_USAGE_MEDIUM</code>, <code>POWER_USAGE_HIGH</code>.
	 */
	public void setPreferredPowerConsumption(int level) {
		powerPref = level;
	}
	
	/**
	 * Sets the preferred cost setting.
	 * <p>Sets whether the requests for location determination is allowed to incur any financial cost
	 * to the user of the terminal.</p>
	 * <p>The default is <code>true</code>, i.e. the method is allowed to cost.</p>
	 * <p>Note that the platform implementation may not always be able to know if a location method
	 * implies cost to the end user or not. If the implementation doesn't know, it MUST assume
	 * that it may cost. When this criteria is set to <code>false</code>, the implementation may only
	 * return a <code>LocationProvider</code> of which it is certain that using it for determining
	 * the location does not cause a per usage cost to the end user.</p>
	 * 
	 * @param costAllowed	<code>false</code> if location determination is not allowed to cost, <code>true</code> if it is allowed to cost
	 */
	public void setCostAllowed(boolean costAllowed) {
		this.costAllowed = costAllowed;
	}
	
	/**
	 * Sets whether the location provider should be able to determine speed and course.
	 * Default is <code>false</code>.
	 * 
	 * @param speedAndCourseRequired	if set to <code>true</code>, the <code>LocationProvider</code> is required to be able to normally determine the speed and course. if set to <code>false</code>, the speed and course are not required.
	 */
	public void setSpeedAndCourseRequired(boolean speedAndCourseRequired) {
		this.speedAndCourseRequired = speedAndCourseRequired;
	}
	
	/**
	 * Sets whether the location provider should be able to determine altitude. Default is <code>false</code>.
	 * 
	 * @param altitudeRequired	if set to <code>true</code>, the <code>LocationProvider</code> is required to be able to normally determine the altitude. If set to <code>false</code>, the altitude is not required.
	 */
	public void setAltitudeRequired(boolean altitudeRequired) {
		this.altitudeRequired = altitudeRequired;
	}
	
	/**
	 * Sets whether the location provider should be able to determine textual address information.
	 * Setting this criteria to <code>true</code> implies that a location provider should be selected
	 * that is capable of providing the textual address information. This does not mean that every
	 * returned location instance necessarily will have all the address information filled in, though.
	 * 
	 * @param addressInfoRequired	if set to <code>true</code>, the <code>LocationProvider</code> is required to be able to normally determine the textual address information. if set to <code>false</code>, the textual address information is not required.
	 */
	public void setAddressInfoRequired(boolean addressInfoRequired) {
		this.addressInfoRequired = addressInfoRequired;
	}
	
	//////////////////////////////////////////////////////////
	// other

	public boolean equals(Object o) {
		if (this.getClass() == o.getClass()) {
			Criteria c = (Criteria) o;
			return (c.isAltitudeRequired() == isAltitudeRequired());
		} else return o.equals(this);
	}
	
	public int hashCode() {
		int code = 0;
		if (altitudeRequired)
			code += 1;
		if (speedAndCourseRequired)
			code += 2;
		// currently only these are relevant
		return code;
	}
}
