/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package javax.microedition.location;

/**
 * This interface represents a listener to events associated with detecting proximity
 * to some registered coordinates. Applications implement this interface and register it with
 * a static method in <code>LocationProvider</code> to obtain notfications when proximity
 * to registered coordinates is detected.
 * <p>This listener is called when the terminal enters the proximity of the registered coordinates.
 * The proximity is defined as the proximity radius around the coordinates combined with
 * the horizontal accuracy of the current sampled location.</p>
 * <p>The listener is called only once when the terminal enters the proximity of the registered
 * coordinates. The registration with these coordinates is cancelled when the listener is called.
 * If the application wants to be notified again about these coordinates, it must re-register
 * the coordinates and the listener.</p> 
 * 
 * @author faerber
 */
public interface ProximityListener {
	
	/**
	 * After registering this listener with the <code>LocationProvider</code>, this method
	 * will be called by the platform when the implementation detects that the current location
	 * of the terminal is within the defined proximity radius of the registered coordinates.
	 * 
	 * @param coordinates	the registered coordinates to which proximity has been detected
	 * @param location	the current location of the terminal
	 */
	public void proximityEvent(Coordinates coordinates, Location location);
	
	/**
	 * Called to notify that the state of the proximity monitoring has changed.
	 * <p>These state changes are delivered to the application as soon as possible after the state
	 * of the monitoring changes.</p>
	 * <p>Regardless of the state, the <code>ProximityListener</code> remains registered
	 * until the application explicitly removes it with <code>LocationProvider.removeProximityListener</code>
	 * or the application exits.</p>
	 * <p>These state changes may be related to state changes of some location providers,
	 * but this is implementation dependent as implementations can freely choose the method used
	 * to implement this proximity monitoring.</p>
	 * 
	 * @param isMonitoringActive	a <code>boolean</code> indicating the new state of the proximity monitoring. <code>true</code> indicates that the proximity monitoring is active and <code>false</code> indicates that the proximity monitoring can't be done currently.
	 */
	public void monitoringStateChanged(boolean isMonitoringActive);

}
