/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package javax.microedition.location;

/**
 * Coordinates along with quality information.
 * 
 * @author faerber
 * @author abiskop
 */
public class QualifiedCoordinates extends Coordinates {
	
	private float horizontalAccuracy;
	private float verticalAccuracy;
	
	/**
	 * Constructs a new <code>QualifiedCoordinates</code> object with the values specified.
	 * The latitude and longitude parameters are expressed in degrees using floating point values.
	 * The degrees are in decimal values (rather than minutes/seconds).
	 * <p>The coordinate values always apply to the WGS84 datum.</p>
	 * <p>The <code>Float.NaN</code> value can be used for altitude to indicate that altitude is not known.</p>
	 * 
	 * @param latitude	the latitude of the location. Valid range: [-90.0, 90.0]
	 * @param longitude	the longitude of the location. Valid range: [-180.0, 180.0)
	 * @param altitude	the altitude of the location in meters, defined as height above WGS84 ellipsoid. <code>Float.NaN</code> can be used to indicate that altitude is not known.
	 * @param horizontalAccuracy	the horizontal accuracy of this location result in meters. <code>Float.NaN</code> can be used to indicate that the accuracy is not known. Must be greater or equal to 0.
	 * @param verticalAccuracy	the vertical accuracy of this location result in meters. <code>Float.NaN</code> can be used to indicate that the accuracy is not known. Must be greater or equal to 0.
	 * 
	 * @throws IllegalArgumentException	if an input parameter is out of the valid range
	 */
	public QualifiedCoordinates(double latitude, double longitude, float altitude,
		float horizontalAccuracy, float verticalAccuracy) {
		super(latitude, longitude, altitude);
		this.horizontalAccuracy = horizontalAccuracy;
		this.verticalAccuracy = verticalAccuracy;
	}
	
	/**
	 * Returns the horizontal accuracy of the location in meters (1-sigma standard deviation). 
	 * A value of <code>Float.NaN</code> means the horizontal accuracy could not be determined. <p>
	 * 
	 * The horizontal accuracy is the RMS (root mean square) of east accuracy (latitudinal error in meters, 1-sigma standard deviation), north accuracy (longitudinal error in meters, 1-sigma). 
	 * 
	 * @return the horizontal accuracy in meters. <code>Float.NaN</code>  if this is not known.
	 */
	public float getHorizontalAccuracy() {
		return horizontalAccuracy;
	}
	
	/**
	 * Returns the accuracy of the location in meters in vertical direction (orthogonal to ellipsoid surface, 1-sigma standard deviation). 
	 * A value of <code>Float.NaN</code> means the vertical accuracy could not be determined.
	 * 
	 * @return the vertical accuracy in meters. <code>Float.NaN</code>  if this is not known.
	 */
	public float getVerticalAccuracy() {
		return verticalAccuracy;
	}
	
	/**
	 * Sets the horizontal accuracy of the location in meters (1-sigma standard deviation). 
	 * A value of <code>Float.NaN</code> means the horizontal accuracy could not be determined.<p>
	 * 
	 * The horizontal accuracy is the RMS (root mean square) of east accuracy (latitudinal error in meters, 1-sigma standard deviation), north accuracy (longitudinal error in meters, 1-sigma). 
	 * 
	 * @param horizontalAccuracy  the horizontal accuracy of this location result in meters. <code>Float.NaN</code> means the horizontal accuracy could not be determined. Must be greater or equal to 0.
	 * @throws IllegalArgumentException  if the parameter is less than 0.
	 */
	public void setHorizontalAccuracy(float horizontalAccuracy) {
		if (horizontalAccuracy < 0) {
			throw new IllegalArgumentException("Horizontal accuracy out of range; must be greater or equal to 0.");
		} else {
			this.horizontalAccuracy = horizontalAccuracy;
		}
	}
	
	/**
	 * Sets the accuracy of the location in meters in vertical direction (orthogonal to ellipsoid surface, 1-sigma standard deviation). 
	 * A value of <code>Float.NaN</code> means the vertical accuracy could not be determined.
	 * 
	 * @param verticalAccuracy  the vertical accuracy of this location result in meters. <code>Float.NaN</code> means the horizontal accuracy could not be determined. Must be greater or equal to 0.
	 * @throws IllegalArgumentException  if the parameter is less than 0.
	 */
	public void setVerticalAccuracy(float verticalAccuracy) {
		if (verticalAccuracy < 0) {
			throw new IllegalArgumentException("Vertical accuracy out of range; must be greater or equal to 0.");
		} else {
			this.verticalAccuracy = verticalAccuracy;
		}
	}
}
