/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loclib;

import javax.microedition.location.*;
import java.util.*;
import java.io.*;
import org.pi4.bluetooth.*;

/**
 * Location provides that determines the position based on Bluetooth MAC addresses.
 * A simple proximity approach is utilized to calculate the position.
 * 
 * @author king
 */
public class BluetoothLocationProvider extends LocationProvider implements StateChangeListener, NeighborsListener { 
	protected Criteria criteria;
	private volatile Vector workers;
	protected Timer timer;
	protected TimerTask timerTask;
	protected org.pi4.bluetooth.InputOutputHandler ioHandler;
	private int state;
	protected LocationListener listener;
	protected int interval, timeout, maxAge;
	
	private volatile double lastLatitude = Double.NaN;
	private volatile double lastLongitude = Double.NaN;
	private volatile float lastHorzAccuracy = Float.NaN;
	private volatile float lastVertAccuracy = Float.NaN;
	private volatile float lastAltitude = Float.NaN;
	private volatile long lastTime = 0;
	
	protected HashMap locData;
	
	private static HashMap instances = new HashMap();
	
	public static LocationProvider getInstance(Criteria criteria) throws LocationException {
		if (criteria == null)
			criteria = new Criteria();
		if (instances.containsKey(criteria)) {
			return (BluetoothLocationProvider) instances.get(criteria);
		} else {
			BluetoothLocationProvider prov = new BluetoothLocationProvider(criteria);
			instances.put(criteria, prov);
			return prov;
		}
	}
	
	private BluetoothLocationProvider(Criteria criteria) {
		this.criteria = criteria;
		workers = new Vector();
		timer = new Timer();
		locData = new HashMap();
		
		try {
			String source = Configuration.getProperty("Bluetooth.source", "device");
			if (source.equals("device")) {
				ioHandler = org.pi4.bluetooth.InputOutputHandler.getInstance();
			} else if (source.equals("file")) {
				ioHandler = org.pi4.bluetooth.InputOutputHandler.getInstance(new File(Configuration.getProperty("Bluetooth.recordedFile", "bluetoothrecords.txt")));
			}
			ioHandler.addStateChangeListener(this);
			ioHandler.addNeighborsListener(this);
			loadLocData();
		} catch (FileNotFoundException fnfe) {
			System.err.println(fnfe.getLocalizedMessage());
		} catch (IOException ioe) {
			System.err.println(ioe.getLocalizedMessage());
		}
	}
	
	private void loadLocData() throws FileNotFoundException, IOException {
		File locationDataFile = new File(Configuration.getProperty("Bluetooth.locationFile", "bluetoothlocationdata.txt"));
		if (locationDataFile.isFile() && locationDataFile.canRead()) {
			BufferedReader reader = new BufferedReader(new InputStreamReader(new FileInputStream(locationDataFile)));
			String line;
			while((line = reader.readLine()) != null) {
				// parse BluetoothAddress
				BluetoothAddress ad = BluetoothAddress.parse(line);

				QualifiedCoordinates co = new QualifiedCoordinates(0,0,0,0,0);

				// read and handle latitude
				line = reader.readLine();
				if (line == null) throw new IOException("Latitude value is not available.");
				co.setLatitude(Double.parseDouble(line));
				
				// read and handle longitude
				line = reader.readLine();
				if (line == null) throw new IOException("Longitude value is not available.");
				co.setLongitude(Double.parseDouble(line));
				
				// read and handle altitude
				line = reader.readLine();
				if (line == null) throw new IOException("Altitude value is not available.");
				co.setAltitude(Float.parseFloat(line));
				
				locData.put(ad, co);
			}
		}
	}
	
	protected void finalize() {
		timer.cancel();
		ioHandler.removeNeighborsListener(this);
		ioHandler.removeStateChangeListener(this);
	}

	public int getState() {
		return state;
	}
	
	public Location getLocation(int timeout) throws LocationException, InterruptedException {
		if ((timeout == 0) || (timeout < -1))
			throw new IllegalArgumentException("Invalid timeout.");
		Worker worker = new Worker();
		synchronized (workers) {
			workers.add(worker);
		}
		Thread thread = new Thread(worker);
		thread.start();
		try {
			if (timeout > 0) {
				thread.join(timeout*1000);
			} else thread.join();
		} catch (InterruptedException ex) {
			System.err.println("InterruptedException: " + ex.getLocalizedMessage());
		}
		synchronized (workers) {
			workers.remove(worker);
		}
		if (getState() == LocationProvider.OUT_OF_SERVICE)
			throw new LocationException("Out of service.");
		if (worker.wasCancelled())
			throw new InterruptedException("Cancelled by reset.");
		if (worker.getLocation() == null)
			throw new LocationException("Timeout.");
		return worker.getLocation();
	}
	
	public synchronized void setLocationListener(LocationListener listener, int interval, int timeout, int maxAge) {
		if (interval < -1)
			throw new IllegalArgumentException("Invalid interval.");
		if (interval != -1) {
			if ((timeout > interval) || (maxAge > interval))
				throw new IllegalArgumentException("Invalid combination of interval, timeout and maxAge.");
			if ((timeout < 1) && (timeout != -1))
				throw new IllegalArgumentException("Invalid timeout.");
			if ((maxAge < 1) && (maxAge != -1))
				throw new IllegalArgumentException("Invalid maxAge.");
		}
		if (timerTask != null) {
			timerTask.cancel();
			timerTask = null;
		}
		this.listener = listener;
		this.interval = (interval == -1) ? 1 : interval;
		this.timeout = ((interval == -1) || (timeout == -1)) ? 5 : timeout;
		this.maxAge = ((interval == -1) || (maxAge == -1)) ? 5 : maxAge;
		if ((listener != null) && (interval != 0)) {
			timerTask = new TimerTask() {
				public void run() {
					notifyLocation();
				}
			};
			timer.scheduleAtFixedRate(timerTask, 0, this.interval*1000);
		}
	}
	
	/**
	 * Notifies the LocationListener about the current location.
	 */
	protected void notifyLocation() {
		Location location;
		if (Math.abs(lastTime - System.currentTimeMillis()) < maxAge*1000) {
			location = new BluetoothLocationData(new QualifiedCoordinates(lastLatitude, lastLongitude,
				lastAltitude, lastHorzAccuracy, lastVertAccuracy), lastTime);
			System.out.println(location);
			System.out.println(lastAltitude);
		} else {
			try {
				location = getLocation(timeout);
			} catch (Exception ex) {
				location = new BluetoothLocationData(new QualifiedCoordinates(Double.NaN, Double.NaN, Float.NaN, Float.NaN, Float.NaN), System.currentTimeMillis());
			}
		}
		if (listener != null)
			listener.locationUpdated(this, location);
	}

	public void reset() {
		synchronized (workers) {
			Iterator it = workers.iterator();
			while (it.hasNext()) {
				Worker worker = (Worker) it.next();
				worker.cancel();
			}
		}
	}
	
	/**
	 * Handles neighbors messages.
	 * 
	 * @param message	neighors message representation
	 */
	public synchronized void handleNeighbors(Neighbors nb) {
		if (nb.size() == 0)
			return;
	
		double latitudeTemp = 0.0;
		double longitudeTemp = 0.0;
		float altitudeTemp = 0.0f;
		float horzAccuracyTemp = 0.0f;
		int counter = 0;
		
		synchronized (nb) {		
			Iterator it = nb.getNeighborBa();
			while (it.hasNext()) {
				// Update location information
				BluetoothAddress ba = (BluetoothAddress) it.next();
				QualifiedCoordinates co = (QualifiedCoordinates) locData.get(ba);
				if (co != null) {
					latitudeTemp += co.getLatitude();
					longitudeTemp += co.getLongitude();
					altitudeTemp += co.getAltitude();
					horzAccuracyTemp += co.getHorizontalAccuracy();
					++counter;
				}
			}
			if (counter != 0) {
				this.lastLatitude = latitudeTemp / counter;
				this.lastLongitude = longitudeTemp / counter;
				this.lastAltitude = altitudeTemp / counter;
				this.lastHorzAccuracy = horzAccuracyTemp / counter; 
				this.lastTime = System.currentTimeMillis();
			}
		}
	}
	
	/**
	 * Handles a state change of the underlying <code>InputOutputHandler</code>.
	 * 
	 * @param newState	the new state of the GPS receiver
	 */
	public void stateChanged(int newState) {
		state = newState;
		if (listener != null) {
			listener.providerStateChanged(this, newState);
		}
	}
	
	/**
	 * Encapsulates the worker thread.
	 * 
	 * @author faerber
	 */
	private class Worker implements Runnable, NeighborsListener {
		
		private volatile Thread thread = null; // the worker thread
		private Location location = null; // the location
		private volatile boolean cancelled = false; // has reset been called?
		private volatile double lastLatitude = Double.NaN;
		private volatile double lastLongitude = Double.NaN;
		private volatile float lastHorzAccuracy = Float.NaN;
		private volatile float lastVertAccuracy = Float.NaN;
		private volatile float lastAltitude = Float.NaN;
		private volatile long lastTime = 0;
		
		/**
		 * Constructs a new worker thread object.
		 */
		public Worker() {
		}
		
		/**
		 * Propagates <code>LocationProvider.reset</code> on to this worker thread.
		 */
		public synchronized void cancel() {
			cancelled = true;
			this.thread.interrupt();
		}
		
		/**
		 * Returns whether the work has been cancelled by a reset call.
		 * 
		 * @return	<code>true</code> if <code>cancel</code> was called, <code>false</code> otherwise
		 */
		public boolean wasCancelled() {
			return cancelled;
		}
		
		protected void finalize() {
			ioHandler.removeNeighborsListener(this);
		}
		
		/**
		 * Gets the location.
		 * 
		 * @return	the location or <code>null</code>
		 */
		public Location getLocation() {
			return location;
		}
		
		/**
		 * Waits for the location to be determined.
		 */
		public void run() {
			// Save current thread for cancel method
			thread = Thread.currentThread();
		
			// Set up listeners
			ioHandler.addNeighborsListener(this);
		
			try {
				// wait for incoming data...
				synchronized (this) {
					this.wait();
				}
			
				// Return Location object (only if there is enough data)
				if ((getState() != LocationProvider.OUT_OF_SERVICE) && (!cancelled)) {
					location = new BluetoothLocationData(new QualifiedCoordinates(lastLatitude, lastLongitude,
						lastAltitude, lastHorzAccuracy, lastVertAccuracy), lastTime);
				}
			} catch (InterruptedException ex) {
				// ignore
			} finally {
				// Unregister listeners before finalizer is called.
				ioHandler.removeNeighborsListener(this);
			}
		}
		
		/**
		 * Handles neighbors messages.
		 * 
		 * @param message	neighors message representation
		 */
		public void handleNeighbors(Neighbors nb) {
			if (nb.size() == 0)
				return;
			
			double latitudeTemp = 0.0;
			double longitudeTemp = 0.0;
			float altitudeTemp = 0.0f;
			float horzAccuracyTemp = 0.0f;
			int counter = 0;
			
			synchronized (nb) {		
				Iterator it = nb.getNeighborBa();
				while (it.hasNext()) {
					// Update location information
					BluetoothAddress ba = (BluetoothAddress) it.next();
					QualifiedCoordinates co = (QualifiedCoordinates) locData.get(ba);
					if (co != null) {
						latitudeTemp += co.getLatitude();
						longitudeTemp += co.getLongitude();
						altitudeTemp += co.getAltitude();
						horzAccuracyTemp += co.getHorizontalAccuracy();
						++counter;
					}
				}
				if (counter != 0) {
					this.lastLatitude = latitudeTemp / counter;
					this.lastLongitude = longitudeTemp / counter;
					this.lastAltitude = altitudeTemp / counter;
					this.lastHorzAccuracy = horzAccuracyTemp / counter; 
					this.lastTime = System.currentTimeMillis();
				}
			}
			synchronized (this) {
				this.notifyAll();
			}
		}
	}
}
