/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loclib;

import java.util.HashMap;
import javax.microedition.location.*;

/**
 * A <code>LocationProvider</code> that switches between GPS and Bluetooth.
 * GPS is preferred over Bluetooth.
 * 
 * @author faerber
 */
public class GpsBluetoothLocationProvider extends LocationProvider implements LocationListener {
	
	private LocationProvider gpsProv;
	private LocationProvider btProv;
	private volatile LocationProvider curProv; // currently used provider
	private LocationListener listener = null; // chained LocationListener
	private int interval = -1, timeout = -1, maxAge = -1;
	
	private static HashMap instances = new HashMap();
	
	public static LocationProvider getInstance(Criteria criteria) throws LocationException {
		if (criteria == null)
			criteria = new Criteria();
		if (instances.containsKey(criteria)) {
			return (GpsBluetoothLocationProvider) instances.get(criteria);
		} else {
			GpsBluetoothLocationProvider prov = new GpsBluetoothLocationProvider(criteria);
			instances.put(criteria, prov);
			return prov;
		}
	}
	
	
	/**
	 * Creates a new <code>GpsBluetoothLocationProvider</code> instance.
	 * 
	 * @throws LocationException	passed on from <code>GpsLocationProvider</code> and <code>BluetoothLocationProvider</code>
	 */
	private GpsBluetoothLocationProvider(Criteria criteria) throws LocationException {
		gpsProv = GpsLocationProvider.getInstance(criteria);
		btProv = BluetoothLocationProvider.getInstance(criteria);
		curProv = (gpsProv.getState() == AVAILABLE) ? gpsProv : btProv;
		gpsProv.setLocationListener(this, 0, -1, -1);
		btProv.setLocationListener(this, 0, -1, -1);
	}
	
	protected void finalize() {
		gpsProv.setLocationListener(null, 0, -1, -1);
		btProv.setLocationListener(null, 0, -1, -1);
	}
	
	public void setLocationListener(LocationListener listener, int interval, int timeout, int maxAge) {
		this.listener = listener;
		this.interval = (listener == null) ? 0 : interval;
		this.timeout = timeout;
		this.maxAge = maxAge;
		curProv.setLocationListener(this, this.interval, this.timeout, this.maxAge);
	}
	
	public Location getLocation(int timeout) throws LocationException, InterruptedException {
		return curProv.getLocation(timeout);
	}
	
	public void reset() {
		gpsProv.reset();
		btProv.reset();
	}
	
	public int getState() {
		return curProv.getState();
	}
	
	/**
	 * Handles a state change notification.
	 * 
	 * @param provider	the <code>LocationProvider</code> instance
	 * @param newState	the new state
	 */
	public void providerStateChanged(LocationProvider provider, int newState) {
		int oldState = getState();
		if (provider == gpsProv) {
			if ((curProv != gpsProv) && (newState == AVAILABLE)) {
				curProv = gpsProv;
				btProv.setLocationListener(this, 0, -1, -1);
				curProv.setLocationListener(this, interval, timeout, maxAge);
			} else if ((curProv == gpsProv) && (newState != AVAILABLE)) {
				curProv = btProv;
				gpsProv.setLocationListener(this, 0, -1, -1);
				curProv.setLocationListener(this, interval, timeout, maxAge);
			}
		}
		if (listener != null) {
			if ((provider == curProv) && (newState != oldState)) {
				listener.providerStateChanged(this, newState);
			}
		}
	}
	
	/**
	 * Handles a location update notification.
	 * 
	 * @param provider	the <code>LocationProvider</code> instance
	 * @param location	the location
	 */
	public void locationUpdated(LocationProvider provider, Location location) {
		if (provider == curProv) {
			if (listener != null)
				listener.locationUpdated(this, location);
		}
	}
}
