/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.loclib;

import org.pi4.nmea0183.*;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;
import java.io.*;

import javax.microedition.location.*;

/**
 * Provides the location based on GPS satellite data.
 * 
 * @author faerber
 */
public class GpsLocationProvider extends LocationProvider
	implements StateChangeListener, GprmcListener, GpggaListener, GpgllListener, GpgsaListener, GpvtgListener {
	
	protected Criteria criteria; // the Location API criteria
	protected org.pi4.nmea0183.InputOutputHandler ioHandler; // NMEA-0183 I/O handler
	private volatile double lastLatitude = Double.NaN;
	private volatile double lastLongitude = Double.NaN;
	private volatile float lastHorzAccuracy = Float.NaN;
	private volatile float lastVertAccuracy = Float.NaN;
	private volatile float lastAltitude = Float.NaN;
	private volatile float lastSpeed = Float.NaN;
	private volatile float lastCourse = Float.NaN;
	private volatile long lastTime = 0;
	private volatile Vector workers; // list of worker thread objects
	private volatile int state = LocationProvider.TEMPORARILY_UNAVAILABLE; // our state
	protected volatile LocationListener listener;
	protected int interval, timeout, maxAge;
	protected Timer timer;
	protected TimerTask timerTask;
	
	private static HashMap instances = new HashMap();
	
	/**
	 * Returns a Singleton instance of this <code>LocationProvider</code>.
	 * 
	 * @return	a <code>GpsLocationProvider</code> instance
	 */
	public static LocationProvider getInstance(Criteria criteria) throws LocationException {
		try {
			if (criteria == null)
				criteria = new Criteria();
			if (instances.containsKey(criteria)) {
				return (GpsLocationProvider) instances.get(criteria);
			} else {
				GpsLocationProvider provider = new GpsLocationProvider(criteria);
				instances.put(criteria, provider);
				return provider;
			}
		} catch (Exception ex) {
			throw new LocationException(ex.getLocalizedMessage());			
		}
	}
	
	/**
	 * Creates a new GPS based location provider.
	 * 
	 * @param criteria	the <code>Criteria</code> on which this location provider is based
	 * 
	 * @throws IOException
	 */
	public GpsLocationProvider(Criteria criteria) throws IOException {
		this.criteria = criteria;
		workers = new Vector();
		timer = new Timer();
		
		String source = Configuration.getProperty("GPS.source", "serial port");
		if (source.equals("serial port")) {
			ioHandler = org.pi4.nmea0183.InputOutputHandler.getInstance(Integer.parseInt(Configuration.getProperty("GPS.serialPort", "3")));
		} else if (source.equals("Bluetooth")) {
			if (Configuration.getProperty("GPS.Bluetooth.address") == null)
				throw new IllegalStateException("No Bluetooth address specified.");
			if (Configuration.getProperty("GPS.Bluetooth.channel") == null)
				throw new IllegalStateException("No RFCOMM channel specified.");
			ioHandler = org.pi4.nmea0183.InputOutputHandler.getInstance(Configuration.getProperty("GPS.Bluetooth.address"), Integer.parseInt(Configuration.getProperty("GPS.Bluetooth.channel")));
		} else if (source.equals("file")) {
			ioHandler = org.pi4.nmea0183.InputOutputHandler.getInstance(new File(Configuration.getProperty("GPS.recordedFile", "gpsrecords.txt")), true);
		} else if (source.equals("device file")) {
			ioHandler = org.pi4.nmea0183.InputOutputHandler.getInstance(new File("/dev/" + Configuration.getProperty("GPS.serialDevice", "ttyS03")), false);
		}
		
		ioHandler.addStateChangeListener(this);
		ioHandler.addGprmcListener(this);
		ioHandler.addGpggaListener(this);
		ioHandler.addGpgllListener(this);
		ioHandler.addGpgsaListener(this);
		ioHandler.addGpvtgListener(this);
	}
	
	protected void finalize() {
		timer.cancel();
		ioHandler.removeGprmcListener(this);
		ioHandler.removeGpggaListener(this);
		ioHandler.removeGpgllListener(this);
		ioHandler.removeGpgsaListener(this);
		ioHandler.removeGpvtgListener(this);
		ioHandler.removeStateChangeListener(this);
	}
	
	public int getState() {
		return state;
	}
	
	public Location getLocation(int timeout) throws LocationException, InterruptedException {
		if ((timeout == 0) || (timeout < -1))
			throw new IllegalArgumentException("Invalid timeout.");
		Worker worker = new Worker();
		synchronized (workers) {
			workers.add(worker);
		}
		Thread thread = new Thread(worker);
		thread.start();
		try {
			if (timeout > 0) {
				thread.join(timeout*1000);
			} else thread.join();
		} catch (InterruptedException ex) {
			System.err.println("InterruptedException: " + ex.getLocalizedMessage());
		}
		synchronized (workers) {
			workers.remove(worker);
		}
		if (getState() == LocationProvider.OUT_OF_SERVICE)
			throw new LocationException("Out of service.");
		if (worker.wasCancelled())
			throw new InterruptedException("Cancelled by reset.");
		if (worker.getLocation() == null)
			throw new LocationException("Timeout.");
		return worker.getLocation();
	}
	
	public synchronized void setLocationListener(LocationListener listener, int interval, int timeout, int maxAge) {
		if (interval < -1)
			throw new IllegalArgumentException("Invalid interval.");
		if (interval != -1) {
			if ((timeout > interval) || (maxAge > interval))
				throw new IllegalArgumentException("Invalid combination of interval, timeout and maxAge.");
			if ((timeout < 1) && (timeout != -1))
				throw new IllegalArgumentException("Invalid timeout.");
			if ((maxAge < 1) && (maxAge != -1))
				throw new IllegalArgumentException("Invalid maxAge.");
		}
		
		if (timerTask != null) {
			timerTask.cancel();
			timerTask = null;
		}
		
		this.listener = listener;
		this.interval		=  (interval == -1) ? 1 : interval;
		this.timeout	= ((interval == -1) || (timeout == -1)) ? 5 : timeout;
		this.maxAge	= ((interval == -1) || (maxAge == -1)) ? 5 : maxAge;
		
		if ((listener != null) && (interval != 0)) {
			timerTask = new TimerTask() {
				public void run() {
					notifyLocation();
				}
			};
			timer.scheduleAtFixedRate(timerTask, 0, this.interval*1000);
		}
	}
	
	/**
	 * Notifies the LocationListener about the current location.
	 */
	protected void notifyLocation() {
		Location location;
		if (Math.abs(lastTime - System.currentTimeMillis()) < maxAge*1000) {
			location = new GpsLocationData(new QualifiedCoordinates(lastLatitude, lastLongitude,
				lastAltitude, lastHorzAccuracy, lastVertAccuracy), lastSpeed, lastCourse, lastTime);
		} else {
			try {
				location = getLocation(timeout);
			} catch (Exception ex) {
				location = new GpsLocationData(new QualifiedCoordinates(Double.NaN, Double.NaN, Float.NaN, Float.NaN, Float.NaN), Float.NaN, Float.NaN, System.currentTimeMillis());
			}
		}
		synchronized (listener) {
			listener.locationUpdated(this, location);
		}
	}
	 
	public void reset() {
		synchronized (workers) {
			Iterator it = workers.iterator();
			while (it.hasNext()) {
				Worker worker = (Worker) it.next();
				worker.cancel();
			}
		}
	}
	
	/**
	 * Handles a state change of the underlying <code>InputOutputHandler</code>.
	 * 
	 * @param newState	the new state of the GPS receiver
	 */
	public void stateChanged(int newState) {
		state = newState;
		if (listener != null) {
			synchronized (listener) {
				listener.providerStateChanged(this, newState);
			}
		}
	}
	
	/**
	 * Handles GPRMC messages.
	 * 
	 * @param message	GPRMC message representation
	 */
	public void handleGprmc(Gprmc message) {
		if (message.getStatus() == 'V') {
			if (state != LocationProvider.TEMPORARILY_UNAVAILABLE) {
				state = LocationProvider.TEMPORARILY_UNAVAILABLE;
				if (listener != null)
					listener.providerStateChanged(this, state);
			}
			return;
		} else { // 'A'
			if (state != LocationProvider.AVAILABLE) {
				state = LocationProvider.AVAILABLE;
				if (listener != null)
					listener.providerStateChanged(this, state);
			}
		}
		lastLatitude		= message.getLatitude().getPosition();
		lastLongitude	= message.getLongitude().getPosition();
		lastSpeed		= (float) message.getSpeedOverGround();
		if (!Float.isNaN(lastSpeed))
			lastSpeed *= 0.51444f;
		lastCourse		= (float) message.getTrackOverGround();
		lastTime			= message.getTimeConstructed();
	}
	
	/**
	 * Handles GPGGA messages.
	 * 
	 * @param message	GPGGA message representation
	 */
	public void handleGpgga(Gpgga message) {
		if (message.getQuality() == 0) {
			if (state != LocationProvider.TEMPORARILY_UNAVAILABLE) {
				state = LocationProvider.TEMPORARILY_UNAVAILABLE;
				if (listener != null)
					listener.providerStateChanged(this, state);
			}
			return;
		} else { // >0
			if (state != LocationProvider.AVAILABLE) {
				state = LocationProvider.AVAILABLE;
				if (listener != null) {
					listener.providerStateChanged(this, state);
				}
			}
		}
		lastLatitude				= message.getLatitude().getPosition();
		lastLongitude			= message.getLongitude().getPosition();
		lastAltitude				= (float) message.getAltitudeAboveGeoid();
		lastHorzAccuracy	= (float) message.getHorizontalDilutionOfPrecision();
		lastTime					= message.getTimeConstructed();
	}
	
	/**
	 * Handles GPGLL messages.
	 * 
	 * @param message	GPGLL message representation
	 */
	public void handleGpgll(Gpgll message) {
		if (message.getStatus() == 'V') {
			if (state != LocationProvider.TEMPORARILY_UNAVAILABLE) {
				state = LocationProvider.TEMPORARILY_UNAVAILABLE;
				if (listener != null)
					listener.providerStateChanged(this, state);
			}
			return;
		} else { // 'A'
			if (state != LocationProvider.AVAILABLE) {
				state = LocationProvider.AVAILABLE;
				if (listener != null)
					listener.providerStateChanged(this, state);
			}
		}
	}
	
	/**
	 * Handles GPGSA messages.
	 * 
	 * @param message	GPGSA message representation
	 */
	public void handleGpgsa(Gpgsa message) {
		lastHorzAccuracy	= (float) message.getHorizontalDilutionOfPrecision();
		lastVertAccuracy		= (float) message.getVerticalDilutionOfPrecision();
		lastTime					= message.getTimeConstructed();
	}
	
	/**
	 * Handles GPVTG messages.
	 * 
	 * @param message	GPVTG message representation
	 */
	public void handleGpvtg(Gpvtg message) {
		lastSpeed = (float) message.getSpeedInKmh();
		if (!Float.isNaN(lastSpeed))
			lastSpeed *= 3.6f;
		lastCourse = (float) message.getTrackOverGround();
	}
	
	/**
	 * Encapsulates the worker thread.
	 * 
	 * @author faerber
	 */
	private class Worker implements Runnable,
		GprmcListener, GpggaListener, GpgsaListener, GpvtgListener {
		
		private volatile Thread thread = null; // the worker thread
		private Location location = null; // the location
		private volatile boolean cancelled = false; // has reset been called?
		private volatile double lastLatitude = Double.NaN;
		private volatile double lastLongitude = Double.NaN;
		private volatile float lastHorzAccuracy = Float.NaN;
		private volatile float lastVertAccuracy = Float.NaN;
		private volatile float lastAltitude = Float.NaN;
		private volatile float lastSpeed = Float.NaN;
		private volatile float lastCourse = Float.NaN;
		private volatile long lastTime = 0;
		
		/**
		 * Constructs a new worker thread object.
		 */
		public Worker() {
		}
		
		/**
		 * Propagates <code>LocationProvider.reset</code> on to this worker thread.
		 */
		public synchronized void cancel() {
			cancelled = true;
			this.thread.interrupt();
		}
		
		/**
		 * Returns whether the work has been cancelled by a reset call.
		 * 
		 * @return	<code>true</code> if <code>cancel</code> was called, <code>false</code> otherwise
		 */
		public boolean wasCancelled() {
			return cancelled;
		}
		
		protected void finalize() {
			ioHandler.removeGpggaListener(this);
			ioHandler.removeGprmcListener(this);
			ioHandler.removeGpgsaListener(this);
			ioHandler.removeGpvtgListener(this);
		}
		
		/**
		 * Gets the location.
		 * 
		 * @return	the location or <code>null</code>
		 */
		public Location getLocation() {
			return location;
		}
		
		/**
		 * Checks whether required data has not yet been collected.
		 * 
		 * @return	<code>false</code> if all data has been collected, <code>true</code> otherwise
		 */
		private boolean dataMissing() {
			return	Double.isNaN(lastLatitude) ||
						Double.isNaN(lastLongitude) ||
						(criteria.isAltitudeRequired() && Float.isNaN(lastAltitude)) ||
						(criteria.isSpeedAndCourseRequired() && (Float.isNaN(lastSpeed) || Float.isNaN(lastCourse)));
		}
		
		/**
		 * Waits for the location to be determined.
		 */
		public void run() {
			// Save current thread for cancel method
			this.thread = Thread.currentThread();
			
			// Set up listeners
			ioHandler.addGpggaListener(this);
			ioHandler.addGprmcListener(this);
			ioHandler.addGpgsaListener(this);
			ioHandler.addGpvtgListener(this);
			
			try {
				// wait for incoming data...
				synchronized (this) {
					this.wait();
				}
				
				// Return Location object (only if there is enough data)
				if ((getState() != LocationProvider.OUT_OF_SERVICE) && (!cancelled)) {
					location = new GpsLocationData(new QualifiedCoordinates(lastLatitude, lastLongitude,
						lastAltitude, lastHorzAccuracy, lastVertAccuracy), lastSpeed, lastCourse, lastTime);
				}
			} catch (InterruptedException ex) {
				// ignore
			} finally {
				// Unregister listeners before finalizer is called.
				ioHandler.removeGpggaListener(this);
				ioHandler.removeGprmcListener(this);
				ioHandler.removeGpgsaListener(this);
				ioHandler.removeGpvtgListener(this);
			}
		}
		
		/**
		 * Notifies the waiting thread that all required data has now been collected.
		 */
		private void notifyComplete() {
			synchronized (this) {
				this.notifyAll();
			}
		}
				
		/**
		 * Handles GPGGA messages.
		 * 
		 * @param message	GPGGA message representation
		 */
		public void handleGpgga(Gpgga message) {
			if (message.getQuality() == 0)
				return;
			this.lastLatitude				= message.getLatitude().getPosition();
			this.lastLongitude			= message.getLongitude().getPosition();
			this.lastAltitude				= (float) message.getAltitudeAboveGeoid();
			this.lastHorzAccuracy	= (float) message.getHorizontalDilutionOfPrecision();
			this.lastTime					= message.getTimeConstructed();
			if (!dataMissing())
				notifyComplete();
		}
		
		/**
		 * Handles GPRMC messages.
		 * 
		 * @param message	GPRMC message representation
		 */
		public void handleGprmc(Gprmc message) {
			if (message.getStatus() == 'V')
				return;
			this.lastLatitude		= message.getLatitude().getPosition();
			this.lastLongitude	= message.getLongitude().getPosition();
			this.lastSpeed		= (float) message.getSpeedOverGround();
			if (!Float.isNaN(this.lastSpeed))
				this.lastSpeed *= 0.51444f;
			this.lastCourse		= (float) message.getTrackOverGround();
			this.lastTime			= message.getTimeConstructed();
			if (!dataMissing())
				notifyComplete();
		}
		
		/**
		 * Handles GPGSA messages.
		 * 
		 * @param message	GPGSA message representation
		 */
		public void handleGpgsa(Gpgsa message) {
			this.lastHorzAccuracy	= (float) message.getHorizontalDilutionOfPrecision();
			this.lastVertAccuracy		= (float) message.getVerticalDilutionOfPrecision();
			this.lastTime					= message.getTimeConstructed();
			if (!dataMissing())
				notifyComplete();
		}
		
		/**
		 * Handles GPVTG messages.
		 * 
		 * @param message	GPVTG message representation
		 */
		public void handleGpvtg(Gpvtg message) {
			this.lastSpeed	= (float) message.getSpeedInKmh();
			if (!Float.isNaN(this.lastSpeed))
				this.lastSpeed *= 3.6f;
			this.lastCourse	= (float) message.getTrackOverGround();
			if (!dataMissing())
				notifyComplete();
		}
	}
}
