/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.HashMap;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
//import javax.bluetooth.*;

/**
 * A Bluetooth based GPS I/O handler.
 * 
 * @author faerber
 */
public class BluetoothInputOutputHandler extends InputOutputHandler {
	
	private String address; // the Bluetooth address
	private int channel; // the RFCOMM channel
	private StreamConnection conn; // the RFCOMM connection
	private static HashMap instances = new HashMap(); // the Singleton instances
	
	/**
	 * Returns the Singleton NMEA-0183 I/O handler instance for the specified Bluetooth address and channel.
	 * 
	 * @param address	the Bluetooth address and channel
	 * 
	 * @return	the instance
	 */
	public static InputOutputHandler getInstance(String address, int channel) throws IOException {
		String key = address + ":" + channel;
		synchronized (instances) {
			if (!instances.containsKey(key)) {
				instances.put(key, new BluetoothInputOutputHandler(address, channel));
			}
			return (InputOutputHandler) instances.get(key);
		}
	}
	
	/**
	 * Creates a new <code>BluetoothInputOutputHandler</code> instance.
	 * 
	 * @param address	the Bluetooth address
	 * @param channel	the RFCOMM channel
	 * 
	 * @throws IOException	if an exception occurs while connecting
	 */
	private BluetoothInputOutputHandler(String address, int channel) throws IOException {
		this.address = address;
		this.channel = channel;
		conn = (StreamConnection) Connector.open("btspp://"+address + ":" + channel);
		init(new BufferedReader(new InputStreamReader(conn.openInputStream())));
	}
	
	/**
	 * Returns the Bluetooth address this I/O handler is currently connected to.
	 * 
	 * @return	the Bluetooth address
	 */
	public String getAddress() {
		return address;
	}
	
	/**
	 * Returns the RFCOMM channel this I/O handler is currently connected to.
	 * 
	 * @return	the channel number
	 */
	public int getChannel() {
		return channel;
	}
	
	public void run() {
		try {
			parse();
		} catch (IOException ex) {
			synchronized (instances) {
				instances.remove(address);
			}
		}
	}
	
	public void close() {
		try {
			super.close();
			conn.close();
		} catch (IOException ex) {
			synchronized (instances) {
				instances.remove(address);
			}
		}
	}
}
