/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

/**
 * Stores information about a coordinate (position, direction)
 * 
 * @author abiskop
 */
public class Coordinate {
	private int degrees;
	private double minutes;
	private char direction;

	/**
	 * Constructor for <code>Coordinate</code>.
	 * 
	 * @param position	the position in degrees and minutes as ddmm.mmmm (latitude) or dddmm.mmmm (longitude)
	 * @param direction    the direction as N,S,E or W
	 * 
	 * @throws IllegalArgumentException	if the dot is missing
	 */
	public Coordinate(String position, char direction) throws IllegalArgumentException {
		int i = position.indexOf('.');
		if (i == -1)
			throw new IllegalArgumentException("Invalid argument.");
		this.degrees = Integer.parseInt(position.substring(0, i - 2));
		this.minutes = Double.parseDouble(position.substring(i-2));
		this.direction = direction;
	}

	/**
	 * Returns the position in degrees and decimal fractions of a degree.
	 * This value is for use with the Location API, with S and W directions represented as negative degrees. 
	 * <p>If direction is not one of N, W, S and E, <code>Double.NaN</code> is returned.</p>
	 *  
	 * @return a geodetic position, represented by full degrees and decimal fractions of a degree
	 * 
	 * @throws IllegalStateException	if <code>direction</code> is not one of N, S, W and E
	 */
	public double getPosition() {
		double coord = this.degrees + (this.minutes / 60);
		switch (this.direction) {
			case 'N':
			case 'E':
				return coord;
			case 'S':
			case 'W':
				return -coord;
			default:
				throw new IllegalStateException("Invalid direction.");
		}
	}
	
	/**
	 * Returns the position in full degrees.
	 * 
	 * @return	(full) degrees
	 */
	public int getDegrees() {
		return this.degrees;
	}
	
	/**
	 * Returns the position remainder in minutes.
	 * 
	 * @return	minutes
	 */
	public double getMinutes() {
		return this.minutes;
	}
	
	/**
	 * Returns the direction. (N = North, S=South, E=East, W=West)
	 * 
	 * @return direction
	 */
	public char getDirection() {
		return this.direction;
	}
	
	/**
	 * Returns a NMEA sentence compliant <code>String</code> representation of this <code>Coordinate</code> object.<p>
	 * 
	 * ddmm.mmmm,X or dddmm.mmmm,X
	 * 
	 * @return stringRepresentation
	 */
	public String toString() {
		StringBuffer sb =  new StringBuffer();
		sb.append(this.degrees);
		if(this.minutes < 10.0) {
			sb.append('0');
		}
		sb.append(this.minutes);
		sb.append(',');
		sb.append(this.direction);
		
		return sb.toString();
	}
	
	/**
	 * Returns a detailed <code>String</code> representation of this <code>Coordinate</code> object.
	 * 
	 * <p>ddmm.mmmm X</p>
	 * 
	 * @return stringRepresentation
	 */
	public String dump() {
		return getPosition() + " " + this.direction;
	}
}
