/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.HashMap;

/**
 * A file based InputOutputHandler.
 * Expects a file with lines consisting of a delay in milliseconds (long), a tab delimiter and the actual NMEA-0183 message.
 * Example:
 * 0	$GPRMC,...
 * 1000	$GPGSV,...
 * The NMEA-0183 message is not modified e.g. to reflect the RMC simulation mode.
 * 
 * @author faerber
 */
public class FileInputOutputHandler extends InputOutputHandler implements org.pi4.loclib.FileInputOutputHandler {
	private final File file; // the input file
	private boolean looping = true; // should the file continue from the beginning at EOF?
	private static HashMap instances = null; // the Singleton instances
	
	public synchronized static InputOutputHandler getInstance(File file) throws FileNotFoundException {
		if (instances == null) {
			instances = new HashMap();
		}
		if (instances.containsKey(file)) {
			return (InputOutputHandler) instances.get(file);
		} else {
			FileInputOutputHandler fioh = new FileInputOutputHandler(file);
			instances.put(file, fioh);
			return fioh;
		}
	}
	
	/**
	 * Creates a new InputOutputHandler for the specified file.
	 * 
	 * @param file	the file
	 * 
	 * @throws FileNotFoundException
	 */
	protected FileInputOutputHandler(File file) throws FileNotFoundException {
		this.file = file;
		looping = org.pi4.loclib.Configuration.getProperty("GPS.recordedFile.loop", "on").equals("on");
		init(new BufferedReader(new InputStreamReader(new FileInputStream(file))));
	}
	
	public void run() {
		try {
			parse();
		} catch (IOException iox) {
			synchronized (instances) {
				instances.remove(file);
			}
		}
	}
	
	/**
	 * Returns the file from which is read.
	 * 
	 * @return the file
	 */
	public File getFile() {
		return file;
	}
	
	protected String readMessage() throws IOException, ParseException {
		try {
			String line;
			synchronized (reader) {
				line = reader.readLine();
				if ((line == null) && looping)
					reader.close();
			}
			if ((line == null) && looping) {
				reader = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
				synchronized (reader) {
					line = reader.readLine();
				}
			}
			if (line == null)
				throw new ParseException("Possibly zero size file?", -1);
			int index = line.indexOf('\t'); // delimiter between sleep time and NMEA-0183 message
			if (index == -1)
				throw new ParseException("No tab delimiter found.", -1);
			long delay = Long.parseLong(line.substring(0, index));
			try {
				Thread.sleep(delay);
			} catch (InterruptedException ex) {
				// ignore - possibly timing incorrect
			}
			return line.substring(index+1);
		} catch (NumberFormatException ex) {
			throw new ParseException(ex.getLocalizedMessage(), 0);
		}
	}
	
	public void close() {		
		super.close();
		synchronized (instances) {
			instances.remove(file);
		}
	}
}
