/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;

/**
 * Representation of a GPGGA (Global Positioning System Fix Data, Time, Position and fix related data for a GPS receiver) data sentence.<p>
 * 
 * Example: $GPGGA,191410,4735.5634,N,00739.3538,E,2,04,4.4,351.5,M,48.0,M,555.5,1001*45<p>
 *
 * GPGGA data format:
 * <table border="1">
 * <tr>
 *  <th>Name:</th>
 *  <th>Example:</th>
 *  <th>Description:</th>
 * </tr>
 * <tr>
 *  <td>Message ID:</td>
 *  <td> $GPGGA</td>
 *  <td>GGA protocol header</td>
 * </tr>
 * <tr>
 *  <td>Time stamp (UTC time):</td>
 *  <td>191410</td>
 *  <td>hhmmss</td>
 * </tr>
 * <tr>
 *  <td>Latitude:</td>
 *  <td>4735.5634</td>
 *  <td>ddmm.mmmm</td>
 * </tr>
 * <tr>
 *  <td>N/S Indicator:</td>
 *  <td>N</td>
 *  <td>N=north or S=south</td>
 * </tr>
 * <tr>
 *  <td>Longitude:</td>
 *  <td>00739.3538</td>
 *  <td>Dddmm.mmmm</td>
 * </tr>
 * <tr>
 *  <td>E/W Indicator:</td>
 *  <td>E</td>
 *  <td>E=east or W=west</td>
 * </tr>
 *  <td>GPS quality indicator:</td>
 *  <td>2</td>
 *  <td>0=not available, 1=GPS, 2=DGPS</td>
  * <tr>
 *  <td>Number of satellites in view:</td>
 *  <td>04</td>
 *  <td>00 - 12</td>
 * </tr>
 * <tr>
 *  <td>Horizontal dilution of precision:</td>
 *  <td>4.4</td>
 *  <td>x.x (meters)</td>
 * </tr>
 * <tr>
 *  <td>Altitude above/below mean-sea-level (geoid):</td>
 *  <td>351.5</td>
 *  <td>x.x</td>
 * </tr>
 * <tr>
 *  <td>Units of altitude:</td>
 *  <td>M</td>
 *  <td>usually meters ('M')</td>
 * </tr>
 * <tr>
 *  <td>Geoidal separation:</td>
 *  <td>48.0</td>
 *  <td>x.x</td>
 * </tr>
 * <tr>
 *  <td>Units of geoidal separation:</td>
 *  <td>M</td>
 *  <td>usually meters ('M')</td>
 * </tr>
 * <tr>
 *  <td>Age of differential GPS data:</td>
 *  <td>555.5</td>
 *  <td>x.x, time in seconds since last SC104 type 1 or 9 update, null field when DGPS is not used</td>
 * </tr>
 * <tr>
 *  <td>Reference station ID:</td>
 *  <td>1001</td>
 *  <td>xxxx, 0000 - 1023</td>
 * </tr>
 * <tr>
 *  <td>Checksum:</td>
 *  <td>*45</td>
 *  <td></td>
 * </tr>
 * </table>
 * 
 * @author abiskop
 */
public class Gpgga extends Nmea0183Sentence {
	private static final String DESCRIPTION = "GPGGA";
	private TimeStamp timeStamp;
	private Coordinate latitude;
	private Coordinate longitude;
	private int quality;
	private int numberOfSatellitesInView;
	private double horizontalDilutionOfPrecision;
	private double altitudeAboveGeoid;
	private String unitsOfAltitudeAboveGeoid;
	private double geoidalSeparation;
	private String unitsOfGeoidalSeparation;
	private double ageOfDifferentialGpsData;
	private int referenceStationId;

	/**
	 * Protected constructor for <code>Gpgga</code>.
	 */
	protected Gpgga(
			TimeStamp timeStamp,
			Coordinate latitude,
			Coordinate longitude,
			int quality,
			int numberOfSatellitesInView,
			double horizontalDilutionOfPrecision,
			double altitudeAboveGeoid,
			String unitsOfAltitudeAboveGeoid,
			double geoidalSeparation,
			String unitsOfGeoidalSeparation,
			double ageOfDifferentialGpsData,
			int referenceStationId) {

		this.timeStamp = timeStamp;
		this.latitude = latitude;
		this.longitude = longitude;
		this.quality = quality;
		this.numberOfSatellitesInView = numberOfSatellitesInView;
		this.horizontalDilutionOfPrecision = horizontalDilutionOfPrecision;
		this.altitudeAboveGeoid = altitudeAboveGeoid;
		this.unitsOfAltitudeAboveGeoid = unitsOfAltitudeAboveGeoid;
		this.geoidalSeparation = geoidalSeparation;
		this.unitsOfGeoidalSeparation = unitsOfGeoidalSeparation;
		this.ageOfDifferentialGpsData = ageOfDifferentialGpsData;
		this.referenceStationId = referenceStationId;
	}

	/**
	 * Requires an array of <code>String</code> as parameter, parses the data contained in the array and returns them as a <code>Gpgga</code> object.
	 *
	 * @param data Gpgga sentence splitted into single fields
	 * 
	 * @return <code>Gpgga</code> instance
	 */
	public static Gpgga parse(String[] data) throws ParseException {
		try {
			return new Gpgga(
			// time stamp
			new TimeStamp(
				Integer.parseInt(data[0].substring(0, 2)),
				Integer.parseInt(data[0].substring(2, 4)),
				Integer.parseInt(data[0].substring(4, 6)),
				Integer.parseInt(data[0].substring(7))),
			// latitude
			new Coordinate(data[1], data[2].charAt(0)),
			// longitude			
			new Coordinate(data[3], data[4].charAt(0)),
			// quality
			Integer.parseInt(data[5]),
			// number of satellites in view
			Integer.parseInt(data[6]),
			// horizontal dilution of precision
			Double.parseDouble(data[7]),
			// altitude above geoid
			(data[8].equals("")) ? 0.0 : Double.parseDouble(data[8]),
			// units of altitude above geoid
			data[9],
			// geoidal separation
			(data[10].equals("")) ? 0.0 : Double.parseDouble(data[10]),
			// units of geodial separation
			data[11],
			// age of differential gps data
			(data[12].equals("")) ? 0.0 : Double.parseDouble(data[12]),
			// reference station id
			Integer.parseInt(data[13]));
		} catch (Exception exception) {
			throw new ParseException("Invalid argument", -1);
		}
	}

	public String getDescription() {
		return DESCRIPTION;
	}

	public int getType() {
		return Nmea0183Sentence.GPGGA_TYPE;
	}

	/**
	 * Returns the time stamp as a <code>TimeStamp</code> object.
	 * 
	 * @return <code>timeStamp</code> instance
	 */
	public TimeStamp getTimeStamp() {
		return this.timeStamp;
	}

	/**
	 * Returns the latitude as a <code>Coordinate</code> object.
	 * 
	 * @return latitude
	 */
	public Coordinate getLatitude() {
		return this.latitude;
	}

	/**
	 * Returns the longitude as a <code>Coordinate</code> object.
	 * 
	 * @return longitude
	 */
	public Coordinate getLongitude() {
		return this.longitude;
	}

	/**
	 * Returns the GPS quality indicator.
	 * 0 - fix not available,
	 * 1 - GPS fix,
	 * 2 - Differential GPS fix 
	 * 3 - GPS PPS Mode, fix valid
	 * 
	 * @return quality
	 */
	public int getQuality() {
		return this.quality;
	}

	/**
	 * Returns the number of satellites that are currently in view (00 - 12).
	 * 
	 * @return numberOfSatellitesInView
	 */
	public int getNumberOfSatellitesInView() {
		return this.numberOfSatellitesInView;
	}

	/**
	 * Returns the horizontal dilution of precision (HDOP).
	 * 
	 * @return horizontalDilutionOfPrecision
	 */
	public double getHorizontalDilutionOfPrecision() {
		return this.horizontalDilutionOfPrecision;
	}

	/**
	 * Returns the altitude above/below geoid (mean sea level).
	 * 
	 * @return altitudeAboveGeoid
	 */
	public double getAltitudeAboveGeoid() {
		return this.altitudeAboveGeoid;
	}

	/**
	 * Returns the unit in which the altitude above geoid is measured. Usually meters ('M').
	 * 
	 * @return unitsOfAltitudeAboveGeoid
	 */
	public String getUnitsOfAltitudeAboveGeoid() {
		return this.unitsOfAltitudeAboveGeoid;
	}

	/**
	 * Returns the geoidal separation. (difference between the WGS-84 earth ellipsoid and mean-sea-level (geoid))
	 * 
	 * @return geoidalSeparation
	 */
	public double getGeoidalSeparation() {
		return this.geoidalSeparation;
	}

	/**
	 * Returns the unit in which the geoidal separation is measured. Usually meters ('M').
	 * 
	 * @return unitsOfGeoidalSeparation
	 */
	public String getUnitsOfGeoidalSeparation() {
		return this.unitsOfGeoidalSeparation;
	}

	/**
	 * Returns the age of differential GPS data, time in seconds since last SC104 type 1 or 9 update, null field when DGPS is not used.
	 * 
	 * @return ageOfDifferentialGpsData
	 */
	public double getAgeOfDifferentialGpsData() {
		return this.ageOfDifferentialGpsData;
	}

	/**
	 * Returns the differential reference station ID (0000-1023).
	 * 
	 * @return referenceStationId
	 */
	public int getReferenceStationId() {
		return this.referenceStationId;
	}

	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.timeStamp.toString());
		msg.append(',');
		msg.append(this.latitude.toString());
		msg.append(',');
		msg.append(this.longitude.toString());
		msg.append(',');
		msg.append(this.quality);
		msg.append(',');
		msg.append(this.numberOfSatellitesInView);
		msg.append(',');
		msg.append(this.horizontalDilutionOfPrecision);
		msg.append(',');
		msg.append(this.altitudeAboveGeoid);
		msg.append(',');
		msg.append(this.unitsOfAltitudeAboveGeoid);
		msg.append(',');
		msg.append(this.geoidalSeparation);
		msg.append(',');
		msg.append(this.unitsOfGeoidalSeparation);
		msg.append(',');
		msg.append(this.ageOfDifferentialGpsData);
		msg.append(',');
		msg.append(this.referenceStationId);
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}

	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Description: ");
		sb.append(DESCRIPTION);
		sb.append("\n");
		sb.append("Time stamp: ");
		sb.append(this.timeStamp.dump());
		sb.append("\n");
		sb.append("Latitude: ");
		sb.append(this.latitude.dump());
		sb.append("\n");
		sb.append("Longitude: ");
		sb.append(this.longitude.dump());
		sb.append("\n");
		sb.append("Quality: ");
		sb.append(this.quality);
		sb.append("\n");
		sb.append("# of satellites in view: ");
		sb.append(this.numberOfSatellitesInView);
		sb.append("\n");
		sb.append("Horizontal dilution: ");
		sb.append(this.horizontalDilutionOfPrecision);
		sb.append("\n");
		sb.append("Altitude above geoid: ");
		sb.append(this.altitudeAboveGeoid);
		sb.append("\n");
		sb.append("Unit (altitude above geoid): ");
		sb.append(this.unitsOfAltitudeAboveGeoid);
		sb.append("\n");
		sb.append("Geoidal separation: ");
		sb.append(this.geoidalSeparation);
		sb.append("\n");
		sb.append("Unit (geoidal separation): ");
		sb.append(this.unitsOfGeoidalSeparation);
		sb.append("\n");
		sb.append("Age of diff. GPS data: ");
		sb.append(this.ageOfDifferentialGpsData);
		sb.append("\n");
		sb.append("Reference station ID: ");
		sb.append(this.referenceStationId);
		sb.append("\n");
		
		return sb.toString();
	}
}
