/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;

/**
 * Representation of a GPGLL (Geographic Position Latitude/Longitude) data sentence.<p>
 * 
 * Example: $GPGLL,4735.5634,N,00739.3538,E,191410,A*4A<p>
 *
 * GPGLL Data Format:
 * <table border="1">
 * <tr>
 *  <th>Name:</th>
 *  <th>Example:</th>
 *  <th>Description:</th>
 * </tr>
 * <tr>
 *  <td>Message ID:</td>
 *  <td> $GPGLL</td>
 *  <td>GLL protocol header</td>
 * </tr>
 * <tr>
 *  <td>Latitude:</td>
 *  <td>4735.5634</td>
 *  <td>ddmm.mmmm</td>
 * </tr>
 * <tr>
 *  <td>N/S Indicator:</td>
 *  <td>N</td>
 *  <td>N=north or S=south</td>
 * </tr>
 * <tr>
 *  <td>Longitude:</td>
 *  <td>00739.3538</td>
 *  <td>Dddmm.mmmm</td>
 * </tr>
 * <tr>
 *  <td>E/W Indicator:</td>
 *  <td>E</td>
 *  <td>E=east or W=west</td>
 * </tr>
 * <tr>
 *  <td>Time stamp (UTC time):</td>
 *  <td>191410</td>
 *  <td>hhmmss</td>
 * </tr>
 * <tr>
 *  <td>Status (receiver warning):</td>
 *  <td>A</td>
 *  <td>A=data valid or V=warning (invalid data)</td>
 * </tr>
 * <tr>
 *  <td>Checksum:</td>
 *  <td>*4A</td>
 *  <td></td>
 * </tr>
 * </table>
 * 
 * @author baumann
 * @author king
 */
public class Gpgll extends Nmea0183Sentence {
	private static final String DESCRIPTION = "GPGLL";
	private Coordinate latitude;
	private Coordinate longitude;
	private TimeStamp timeStamp;
	private char status;

	/**
	  * Protected constructor for <code>Gpgll</code>.
	  */
	protected Gpgll(
			Coordinate latitude,
			Coordinate longitude,
			TimeStamp timeStamp,
			char status) {
				
		this.latitude = latitude;
		this.longitude = longitude;
		this.timeStamp = timeStamp;
		this.status = status;
	}

	/**
	 * Requires an array of <code>String</code> as parameter, parses the data contained
	 *  in the array and  returns them as <code>Gpgll</code> object.
	 *
	 * @param data	Gpgll sentence splitted into single fields
	 * 
	 * @return <code>Gpgll</code> instance
	 */
	public static Gpgll parse(String[] data) throws ParseException {
		try {
			return new Gpgll(
				// latitude
				new Coordinate(data[0], data[1].charAt(0)),
				// longitude
				new Coordinate(data[2], data[3].charAt(0)),
				// timestamp
				new TimeStamp(
					Integer.parseInt(data[4].substring(0, 2)),
					Integer.parseInt(data[4].substring(2, 4)),
					Integer.parseInt(data[4].substring(4, 6)),
					Integer.parseInt(data[4].substring(7))),
				// status
				data[5].charAt(0));
		} catch (Exception exception) {
			throw new ParseException("Invalid argument", -1);
		}
	}

	/**
	 * Returns the latitude as a <code>Coordinate</code> object.
	 * 
	 * @return latitude
	 */
	public Coordinate getLatitude() {
		return this.latitude;
	}

	/**
	 * Returns the longitude as a <code>Coordinate</code> object.
	 * 
	 * @return longitude
	 */
	public Coordinate getLongitude() {
		return this.longitude;
	}

	/**
	 * Returns the time stamp as a <code>TimeStamp</code> object.
	 * 
	 * @return timeStamp 
	 */
	public TimeStamp getTimeStamp() {
		return this.timeStamp;
	}

	/**
	 * Returns the status
	 * A - valid,
	 * V - invalid
	 * 
	 * @return status 
	 */
	public char getStatus() {
		return this.status;
	}

	public String getDescription() {
		return DESCRIPTION;
	}

	public int getType() {
		return Nmea0183Sentence.GPGLL_TYPE;
	}

	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.latitude.toString());
		msg.append(',');
		msg.append(this.longitude.toString());
		msg.append(',');
		msg.append(this.timeStamp);
		msg.append(',');
		msg.append(this.status);
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}

	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Description: ");
		sb.append(DESCRIPTION);
		sb.append("\n");
		sb.append("Latitude: ");
		sb.append(this.latitude.dump());
		sb.append("\n");
		sb.append("Longitude: ");
		sb.append(this.longitude.dump());
		sb.append("\n");
		sb.append("Time stamp: ");
		sb.append(this.timeStamp.dump());
		sb.append("\n");
		sb.append("Status: ");
		sb.append(this.status);
		sb.append("\n");
		
		return sb.toString();
	}
}
