/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Representation of a GPGSA (satellites active) data sentence.<p>
 * 
 * Example: $GPGSA,A,3,,,,15,17,18,23,,,,,,4.7,4.4,1.5*3F<p>
 *
 * GPGSA data format:
 * <table border="1">
 * <tr>
 *  <th>Name:</th>
 *  <th>Example:</th>
 *  <th>Description:</th>
 * </tr>
 * <tr>
 *  <td>Message ID:</td>
 *  <td> $GPGSA</td>
 *  <td>GSA protocol header</td>
 * </tr>
 * <tr>
 *  <td>Selection mode:</td>
 *  <td>A</td>
 *  <td>auto-select 2D/3D</td>
 * </tr>
 * <tr>
 *  <td>Mode:</td>
 *  <td>3</td>
 *  <td>3=3D fix, 2=2D fix, 1=fix not available</td>
 * </tr>
 * <tr>
 *  <td>Satellite IDs:</td>
 *  <td>,,,,15,17,18,23,,,,,,</td>
 *  <td>PRN numbers of up to 12 satellites in use</td>
 * </tr>
 * <tr>
 *  <td>Position dilution of precision (PDOP):</td>
 *  <td>4.7</td>
 *  <td>x.x</td>
 * </tr>
 * <tr>
 *  <td>Horizontal dilution of precision (HDOP):</td>
 *  <td>4.4</td>
 *  <td>x.x</td>
 * </tr>
 *  <td>Vertical dilution of precision (VDOP):</td>
 *  <td>1.5</td>
 *  <td>x.x</td>
 * <tr>
 *  <td>Checksum:</td>
 *  <td>*3F</td>
 *  <td></td>
 * </tr>
 * </table>
 * 
 * @author abiskop
 */
public class Gpgsa extends Nmea0183Sentence {
	private static final String DESCRIPTION = "GPGSA";
	private char selectionMode;
	private int mode;
	private Vector satellitesUsed; //of String
	private double positionDilutionOfPrecision;
	private double horizontalDilutionOfPrecision;
	private double verticalDilutionOfPrecision;
	
	/**
	 * Protected constructor for <code>Gpgsa</code>. 
	 */
	protected Gpgsa(
			char selectionMode,
			int mode,
			Vector satellitesUsed,
			double positionDilutionOfPrecision,
			double horizontalDilutionOfPrecision,
			double verticalDilutionOfPrecision) {
		
		this.selectionMode = selectionMode;
		this.mode = mode;
		this.satellitesUsed = satellitesUsed;
		this.positionDilutionOfPrecision = positionDilutionOfPrecision;
		this.horizontalDilutionOfPrecision = horizontalDilutionOfPrecision;
		this.verticalDilutionOfPrecision = verticalDilutionOfPrecision;
	}
	
	/**
	 * Extracts the IDs of the satellites used and returns them as a vector
	 * 
	 * @param data	Gpgsa sentence splitted into single fields
	 * @param offset	position of first satellite-ID in data array
	 * 
	 * @return satellitesUsed
	 * 
	 * @see <a href=http://nmea.de/nmea0183datensaetze.html>http://nmea.de/nmea0183datensaetze.html</a>
	 * @see <a href=http://www.kowoma.de/gps/zusatzerklaerungen/NMEA.htm>http://www.kowoma.de/gps/zusatzerklaerungen/NMEA.htm</a>
	 */
	private static Vector extractSatellitesUsed(String[] data, int offset) {
		final int MAX_NUMBER_OF_SATELLITES = 12; // maximum number of satellites in use
		Vector v = new Vector(0, 1);
		for (int i = 0; i < MAX_NUMBER_OF_SATELLITES; i++) {
			if (!data[offset + i].equals(""))
				v.addElement(data[offset + i]);
		}
		return v;
	}
	
	/**
	 * Requires an array of <code>String</code> as parameter, parses the data contained in the array and returns them as a <code>Gpgsa</code> object.
	 *
	 * @param data <code>Gpgsa</code> sentence splitted into single fields
	 * 
	 * @return <code>Gpgsa</code> instance
	 */
	public static Gpgsa parse(String[] data) throws ParseException {
		try {
			return new Gpgsa( 
				// selection mode
				data[0].charAt(0),
				// mode
				Integer.parseInt(data[1]),
				// satellites used
				extractSatellitesUsed(data, 2),
				// position DOP
				Double.parseDouble(data[14]),
				// horizontal DOP
				Double.parseDouble(data[15]),
				// vertical DOP
				Double.parseDouble(data[16]));
		} catch (Exception exception) {
			throw new ParseException("Invalid argument", -1);
		}
	}
	
	public String getDescription() {
		return DESCRIPTION;
	}
	
	public int getType() {
		return Nmea0183Sentence.GPGSA_TYPE;
	}
	
	/**
	 * Returns the selection mode (A=auto)
	 * 
	 * @return selectionMode
	 */
	public char getSelectionMode() {
		return this.selectionMode;
	}
	
	/**
	 * Returns the mode for positioning (3=3D fix, 2=2D fix, 1=fix not available)
	 * 
	 * @return mode
	 */
	public int getMode() {
		return this.mode;
	}
	
	/**
	 * Returns the IDs of the satellites used as a <code>Vector</code>.
	 * 
	 * @return satellitesUsed
	 */
	public Vector getSatellitesUsed() {
		return this.satellitesUsed;
	}
	
	/**
	 * Returns the position dilution of precision (PDOP).
	 *  
	 * @return positionDilutionOfPrecision
	 */
	public double getPositionDilutionOfPrecision() {
		return this.positionDilutionOfPrecision;
	}
	
	/**
	 * Returns the horizontal dilution of precision (HDOP).
	 * 
	 * @return horizontalDilutionOfPrecision
	 */
	public double getHorizontalDilutionOfPrecision() {
		return this.horizontalDilutionOfPrecision;
	}
	
	/**
	 * Returns the vertical dilution of precision (VDOP).
	 * 
	 * @return verticalDilutionOfPrecision
	 */
	public double getVerticalDilutionOfPrecision() {
		return this.verticalDilutionOfPrecision;
	}
	
	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.selectionMode);
		msg.append(',');
		msg.append(this.mode);
		msg.append(',');
		Enumeration e = this.satellitesUsed.elements();
		while (e.hasMoreElements()) {
			msg.append((String) e.nextElement());
			msg.append(',');
		}
		for (int i = satellitesUsed.size(); i < 12; i++)
			msg.append(',');
		msg.append(this.positionDilutionOfPrecision);
		msg.append(',');
		msg.append(this.horizontalDilutionOfPrecision);
		msg.append(',');
		msg.append(this.verticalDilutionOfPrecision);
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}
	
	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Description: ");
		sb.append(DESCRIPTION);
		sb.append("\n");
		sb.append("Selection mode: ");
		sb.append(this.selectionMode);
		sb.append("\n");
		sb.append("Mode: ");
		sb.append(this.mode);
		sb.append("\n");
		sb.append("Satellites used for fix: ");
		sb.append("\n");
		
		// the vector with the satellite IDs must be converted to a String
		Enumeration e = this.satellitesUsed.elements();
		while (e.hasMoreElements()) {
			sb.append((String) e.nextElement());
			sb.append("\n");
		}
		
		sb.append("Position DOP: ");
		sb.append(this.positionDilutionOfPrecision);
		sb.append("\n");
		sb.append("Horizontal DOP: ");
		sb.append(this.horizontalDilutionOfPrecision);
		sb.append("\n");
		sb.append("Vertical DOP: ");
		sb.append(this.verticalDilutionOfPrecision);
		sb.append("\n");
		
		return sb.toString();
	}
}
