/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;
import java.util.Enumeration;
import java.util.Vector;

/**
 * Representation of a GPGSV (satellites in view) data sentence.<p>
 * 
 * Example: $GPGSV,2,1,08,02,59,282,00,03,42,287,00,06,16,094,00,15,80,090,48*79<p>
 *
 * GPGSV data format:
 * <table border="1">
 * <tr>
 *  <th>Name:</th>
 *  <th>Example:</th>
 *  <th>Description:</th>
 * </tr>
 * <tr>
 *  <td>Message ID:</td>
 *  <td> $GPGSV</td>
 *  <td>GSV protocol header</td>
 * </tr>
 * <tr>
 *  <td>Total number of messages:</td>
 *  <td>2</td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>Message number:</td>
 *  <td>1</td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>Satellites in view:</td>
 *  <td>08</td>
 *  <td>00 -12</td>
 * </tr>
 * <tr>
 *  <td>1) Satellite number (satellite ID):</td>
 *  <td>02</td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>2) Elevation:</td>
 *  <td>59</td>
 *  <td>degrees</td>
 * </tr>
 *  <td>3) Azimuth:</td>
 *  <td>282</td>
 *  <td>degrees to true North</td>
  * <tr>
 *  <td>4) Signal-to-noise ratio (SNR):</td>
 *  <td>00</td>
 *  <td>dB</td>
 * </tr>
 * <tr>
 *  <td>- What follows are more satellite infos like 1) - 4) -</td>
 *  <td></td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>- There can be up to 4 satellite infos per GPGSV sentence -</td>
 *  <td></td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>Checksum:</td>
 *  <td>*79</td>
 *  <td></td>
 * </tr>
 * </table>
 * 
 * @author abiskop
 */
public class Gpgsv extends Nmea0183Sentence {
	private static final String DESCRIPTION = "GPGSV";
	private int totalNumberOfMessages;
	private int messageNumber;
	private int numberOfSatellitesInView;
	private Vector satelliteInfo; //of SatelliteInfo
	
	/**
	 * Protected constructor for <code>Gprmc</code>. 
	 */
	protected Gpgsv(
			int totalNumberOfMessages,
			int messageNumber,
			int numberOfSatellitesInView,
			Vector satelliteInfo) {
			
		this.totalNumberOfMessages = totalNumberOfMessages;
		this.messageNumber = messageNumber;
		this.numberOfSatellitesInView = numberOfSatellitesInView;
		this.satelliteInfo = satelliteInfo;
	}
	
	/**
	 * Extracts one satellite info block from a <code>String</code> array and
	 * returns it as a <code>SatelliteInfo</code> object.<p>
	 * One satellite info block contains information about<p>
	 * - ID of the satellite<p>
	 * - elevation<p>
	 * - azimuth<p>
	 * - signal-to-noise ratio.<p>
	 * 
	 * @param data <code>Gpgsv</code> sentence splitted into single fields
	 * @param offset	index of the first element of the reqiured satellite info data block, namely the ID of the required satellite in the <code>Gpgsv</code> sentence.
	 * 
	 * @return satelliteInfo
	 * 
	 * @see org.pi4.nmea0183.SatelliteInfo
	 */
	private static SatelliteInfo extractSatelliteInfo(String[] data, int offset) {
		return new SatelliteInfo(
								Integer.parseInt(data[offset]),
								Integer.parseInt(data[offset + 1]),
								Integer.parseInt(data[offset + 2]),
								(data[offset + 3].equals("")) ? SatelliteInfo.SNR_NOT_VALID : Integer.parseInt(data[offset + 3]));
	}
	
	/**
	 * Extracts all satellite info blocks from a <code>String</code> array and returns it
	 * in a <code>Vector</code> of <code>SatelliteInfo</code>.<p>
	 * 
	 * There can be 0 to 4 "info blocks" in one GPGSV sentence, BUT:<p>
	 * The commata (',') stay in place, independent from the number of info blocks.
	 * 
	 * @param data	<code>Gpgsv</code> sentence splitted into single fields
	 * @param offset	index of the first element of the first satellite info data block, namely the ID of the first satellite in the <code>Gpgsv</code> sentence.
	 * 
	 * @return satelliteInfoVector
	 * 
	 * @throws ParseException
	 * 
	 * @see org.pi4.nmea0183.SatelliteInfo
	 * @see <a href=http://nmea.de/nmea0183datensaetze.html>http://nmea.de/nmea0183datensaetze.html</a>
	 * @see <a href=http://www.kowoma.de/gps/zusatzerklaerungen/NMEA.htm>http://www.kowoma.de/gps/zusatzerklaerungen/NMEA.htm</a>
	 */
	private static Vector extractAllSatelliteInfos(String[] data, int offset) throws ParseException {
		final int DATABLOCK_LENGTH = 4; // length of one "info block" containing sat ID, elev., azim., SNR
		final int MAX_NUMBER_OF_BLOCKS = 4; // maximum number of data blocks (usually 4)
		Vector v = new Vector(0, 1);
		for (int i = offset; i < data.length; i += DATABLOCK_LENGTH) {
			if (i + DATABLOCK_LENGTH - 1 >= data.length)
				throw new ParseException("Invalid number of arguments.", i);
			v.addElement(extractSatelliteInfo(data, i));
		}
		return v;
	}
	
	/**
	 * Requires an array of <code>String</code> as parameter, parses the data contained in the array and returns them as a <code>Gpgsv</code> object
	 *
	 * @param data <code>Gpgsv</code> sentence splitted into single fields
	 * 
	 * @return <code>Gpgsv</code> instance
	 */
	public static Gpgsv parse(String[] data) throws ParseException {
		try {
			return new Gpgsv(
				// total number of messages
				Integer.parseInt(data[0]),
				// message number
				Integer.parseInt(data[1]),
				// number of satellites in view
				Integer.parseInt(data[2]),
				// satellite infos
				extractAllSatelliteInfos(data, 3));
		} catch (Exception exception) {
			throw new ParseException("Invalid argument", -1);		
		}
	}

	public String getDescription() {
		return DESCRIPTION;
	}
	
	public int getType() {
		return Nmea0183Sentence.GPGSV_TYPE;
	}
	
	/**
	 * Returns the total number of GPGSV messages.
	 * 
	 * @return totalNumberOfMessages
	 */
	public int getTotalNumberOfMessages() {
		return this.totalNumberOfMessages;
	}
	
	/**
	 * Returns the number of this message.
	 *  
	 * @return messageNumber
	 */
	public int getMessageNumber() {
		return this.messageNumber;
	}
	
	/**
	 * Returns the number of satellites in view.
	 * 
	 * @return numberOfSatellitesInView
	 */
	public int getNumberOfSatellitesInView() {
		return this.numberOfSatellitesInView;
	}
	
	/**
	 * Returns a <code>Vector</code> containing <code>SatelliteInfo</code> objects.
	 * 
	 * @return satelliteInfo
	 * 
	 * @see org.pi4.nmea0183.SatelliteInfo
	 */
	public Vector getSatelliteInfo() {
		return this.satelliteInfo;
	}
	
	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.totalNumberOfMessages);
		msg.append(',');
		msg.append(this.messageNumber);
		msg.append(',');
		msg.append(this.numberOfSatellitesInView);
		msg.append(',');
		Enumeration e = this.satelliteInfo.elements();
		while (e.hasMoreElements()) {
			msg.append(e.nextElement().toString());
			if (e.hasMoreElements()) msg.append(',');
		}
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}

	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Description: ");
		sb.append(DESCRIPTION);
		sb.append("\n");
		sb.append("Total number of messages: ");
		sb.append(this.totalNumberOfMessages);
		sb.append("\n");
		sb.append("Message number: ");
		sb.append(this.messageNumber);
		sb.append("\n");
		sb.append("Number of sats in view: ");
		sb.append(this.numberOfSatellitesInView);
		sb.append("\n");
		
		// the vector with the satellite infos must be converted to a String
		Enumeration e = this.satelliteInfo.elements();
		SatelliteInfo satInfo;
		while (e.hasMoreElements()) {
			satInfo = (SatelliteInfo) e.nextElement();
			sb.append(satInfo.dump());
			sb.append("\n");
		}
		
		return sb.toString();
	}
}
