/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;

/**
 * Representation of a GPRMC (Recommended Minimum Navigation Information) data sentence.<p>
 * 
 * Example: $GPRMC,191410,A,4735.5634,N,00739.3538,E,0.0,0.0,181102,0.4,E,A*19<p>
 *
 * GPRMC data format:
 * <table border="1">
 * <tr>
 *  <th>Name:</th>
 *  <th>Example:</th>
 *  <th>Description:</th>
 * </tr>
 * <tr>
 *  <td>Message ID:</td>
 *  <td> $GPRMC</td>
 *  <td>RMC protocol header</td>
 * </tr>
 * <tr>
 *  <td>Time stamp (UTC time):</td>
 *  <td>191410</td>
 *  <td>hhmmss</td>
 * </tr>
 * <tr>
 *  <td>Status (receiver warning):</td>
 *  <td>A</td>
 *  <td>A=data valid or V=warning (invalid data)</td>
 * </tr>
 * <tr>
 *  <td>Latitude:</td>
 *  <td>4735.5634</td>
 *  <td>ddmm.mmmm</td>
 * </tr>
 * <tr>
 *  <td>N/S Indicator:</td>
 *  <td>N</td>
 *  <td>N=north or S=south</td>
 * </tr>
 * <tr>
 *  <td>Longitude:</td>
 *  <td>00739.3538</td>
 *  <td>Dddmm.mmmm</td>
 * </tr>
 * <tr>
 *  <td>E/W Indicator:</td>
 *  <td>E</td>
 *  <td>E=east or W=west</td>
 * </tr>
 *  <td>Speed over ground:</td>
 *  <td>0.0</td>
 *  <td>x.x, knots</td>
  * <tr>
 *  <td>Track over ground:</td>
 *  <td>0.0</td>
 *  <td>x.x, degrees true</td>
 * </tr>
 * <tr>
 *  <td>Date:</td>
 *  <td>181102</td>
 *  <td>ddmmyy</td>
 * </tr>
 * <tr>
 *  <td>Magnetic variation:</td>
 *  <td>0.4</td>
 *  <td>x.x, degrees</td>
 * </tr>
 * <tr>
 *  <td>Magnetic variation E/W Indicator:</td>
 *  <td>E</td>
 *  <td>E=east or W=west</td>
 * </tr>
 * <tr>
 *  <td>Checksum:</td>
 *  <td>*19</td>
 *  <td></td>
 * </tr>
 * </table>
 * 
 * @author abiskop
 */
public class Gprmc extends Nmea0183Sentence {
	private static final String DESCRIPTION = "GPRMC";
	private TimeStamp timeStamp;
	private char status;
	private Coordinate latitude;
	private Coordinate longitude;
	private double speedOverGround;
	private double trackOverGround;
	private Date date;
	private double magneticVariation;
	private char magneticVariationEastWestIndicator;
	
	/**
	 * Protected constructor for <code>Gprmc</code>. 
	 */
	protected Gprmc(
			TimeStamp timeStamp,
			char status,
			Coordinate latitude,
			Coordinate longitude,
			double speedOverGround,
			double trackOverGround,
			Date date,
			double magneticVariation,
			char magneticVariationEastWestIndicator) {
										
		this.timeStamp = timeStamp;
		this.status = status;
		this.latitude = latitude;
		this.longitude = longitude;
		this.speedOverGround = speedOverGround;
		this.trackOverGround = trackOverGround;
		this.date = date;
		this.magneticVariation = magneticVariation;
		this.magneticVariationEastWestIndicator = magneticVariationEastWestIndicator;
	}
	
	/**
	 * Requires an array of <code>String</code> as parameter, parses the data contained in the array and returns them as a <code>Gprmc</code> object.
	 *
	 * @param data	Gprmc sentence splitted into single fields
	 * 
	 * @return <code>Ggprmc</code> instance
	 */
	public static Gprmc parse(String[] data) throws ParseException {
		try {
			return new Gprmc(
				// time stamp
				new TimeStamp(
					Integer.parseInt(data[0].substring(0, 2)),
					Integer.parseInt(data[0].substring(2, 4)),
					Integer.parseInt(data[0].substring(4, 6)),
					Integer.parseInt(data[0].substring(7))),
				// status
				data[1].charAt(0),
				// latitude
				new Coordinate(data[2], data[3].charAt(0)),
				// longitude
				new Coordinate(data[4], data[5].charAt(0)),
				// speed over ground
				Double.parseDouble(data[6]),
				// track over ground
				(data[7].equals("")) ? 0.0 : Double.parseDouble(data[7]),
				// date
				new Date(
					Integer.parseInt(data[8].substring(0, 2)),
					Integer.parseInt(data[8].substring(2, 4)),
					Integer.parseInt(data[8].substring(4, 6))),
				// magnetic variation
				(data[9].equals("")) ? 0.0 : Double.parseDouble(data[9]),
				// magnetic variation E/W indicator
				(data[10].equals("")) ? '\0' : data[10].charAt(0));
		} catch (Exception exception) {
			throw new ParseException("Invalid Argument", -1);
		}
	}
	
	public String getDescription() {	
		return DESCRIPTION;
	}
	
	public int getType() {
		return Nmea0183Sentence.GPRMC_TYPE;
	}
	
	/**
	 * Returns the time stamp as a <code>timeStamp</code> object.
	 * 
	 * @return <code>timeStamp</code> instance 
	 */
	public TimeStamp getTimeStamp() {
		return this.timeStamp;
	}
	
	/**
	 * Returns the status (A = data OK, V = navigation receiver warning).
	 * 
	 * @return status
	 */
	public char getStatus() {
		return this.status;
	}
	
	/**
	 * Returns the latitude as a <code>Coordinate</code> object.
	 * 
	 * @return latitude
	 */
	public Coordinate getLatitude() {
		return this.latitude;
	}
	
	/**
	 * Returns the longitude as a <code>Coordinate</code> object.
	 * 
	 * @return longitude
	 */
	public Coordinate getLongitude() {	
		return this.longitude;
	}
	
	/**
	 * Returns the speed over ground in knots.
	 * 
	 * @return speedOverGround
	 */
	public double getSpeedOverGround() {
		return this.speedOverGround;
	}
	
	/**
	 * Returns the track over ground in degrees.
	 * 
	 * @return trackOverGround
	 */
	public double getTrackOverGround() {
		return this.trackOverGround;
	}
	
	/**
	 * Returns the date as a <code>Date</code> object.
	 * 
	 * @return date
	 */
	public Date getDate() {
		return this.date;
	}
	
	/**
	 * Returns the magnetic variation.
	 * 
	 * @return magneticVariation
	 */
	public double getMagneticVariation() {
		return this.magneticVariation;
	}
	
	/**
	 * Returns the E/W indicator for magnetic variation.
	 * 
	 * @return magneticVariationEastWestIndicator
	 */
	public char getMagneticVariationEastWestIndicator() {
		return this.magneticVariationEastWestIndicator;
	}
	
	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.timeStamp.toString());
		msg.append(',');
		msg.append(this.status);
		msg.append(',');
		msg.append(this.latitude.toString());
		msg.append(',');
		msg.append(this.longitude.toString());
		msg.append(',');
		msg.append(this.speedOverGround);
		msg.append(',');
		msg.append(this.trackOverGround);
		msg.append(',');
		msg.append(this.date.toString());
		msg.append(',');
		msg.append(this.magneticVariation);
		msg.append(',');
		if (this.magneticVariationEastWestIndicator != '\0')
			msg.append(this.magneticVariationEastWestIndicator);
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}
	
	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Description: ");
		sb.append(DESCRIPTION);
		sb.append("\n");
		sb.append("Status: ");
		sb.append(this.status);
		sb.append("\n");
		sb.append("Time stamp: ");
		sb.append(this.timeStamp.dump());
		sb.append("\n");
		sb.append("Latitude: " + this.latitude.dump());
		sb.append("\n");
		sb.append("Longitude: ");
		sb.append(this.longitude.dump());
		sb.append("\n");
		sb.append("Speed over ground: ");
		sb.append(this.speedOverGround);
		sb.append("\n");
		sb.append("Track over ground: ");
		sb.append(this.trackOverGround);
		sb.append("\n");
		sb.append("Date: ");
		sb.append(this.date.dump());
		sb.append("\n");
		sb.append("Magnetic variation: ");
		sb.append(this.magneticVariation);
		sb.append("\n");
		
		return sb.toString();
	}
}
