/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;

/**
 * Representation of a GPVTG (track made good and ground speed) data sentence.<p>
 * 
 * Example: $GPVTG,0.0,T,359.6,M,0.0,N,0.0,K*47<p>
 *
 * GPGSV data format:
 * <table border="1">
 * <tr>
 *  <th>Name:</th>
 *  <th>Example:</th>
 *  <th>Description:</th>
 * </tr>
 * <tr>
 *  <td>Message ID:</td>
 *  <td> $GPVTG</td>
 *  <td>VTG protocol header</td>
 * </tr>
 * <tr>
 *  <td>Track over ground (true):</td>
 *  <td>0.0</td>
 *  <td>x.x</td>
 * </tr>
 * <tr>
 *  <td>True:</td>
 *  <td>T</td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>Track over ground (magnetic):</td>
 *  <td>359.6</td>
 *  <td>x.x</td>
 * </tr>
 * <tr>
 *  <td>Magnetic:</td>
 *  <td>M</td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>Speed over ground in knots:</td>
 *  <td>0.0</td>
 *  <td>x.x</td>
 * </tr>
 *  <td>Knots:</td>
 *  <td>N</td>
 *  <td></td>
  * <tr>
 *  <td>Speed over ground in km/h:</td>
 *  <td>0.0</td>
 *  <td>x.x</td>
 * </tr>
 * <tr>
 *  <td>Km/h:</td>
 *  <td>K</td>
 *  <td></td>
 * </tr>
 * <tr>
 *  <td>Checksum:</td>
 *  <td>47</td>
 *  <td></td>
 * </tr>
 * </table>
 * 
 * @author abiskop
 */
public class Gpvtg extends Nmea0183Sentence {
	private static final String DESCRIPTION = "GPVTG";
	private double trackOverGroundTrue;
	private double trackOverGroundMagnetic;
	private double speedInKnots;
	private double speedInKmh;
	
	protected Gpvtg(
			double trackOverGroundTrue,
			double trackOverGroundMagnetic,
			double speedInKnots,
			double speedInKmh) {
	
		this.trackOverGroundTrue = trackOverGroundTrue;
		this.trackOverGroundMagnetic = trackOverGroundMagnetic;
		this.speedInKnots = speedInKnots;
		this.speedInKmh = speedInKmh;
	}
	
	/**
	 * Requires an array of <code>String</code> as parameter, parses the data contained in the array and returns theam as <code>Gpvtg</code> object.
	 *
	 * @param data Gpvtg sentence splitted into single fields
	 * 
	 * @return <code>Gpvtg</code> instance
	 */
	public static Gpvtg parse(String[] data) throws ParseException {
		try {
			return new Gpvtg(
				// track over ground, true
				(data[0].equals("")) ? 0.0 : Double.parseDouble(data[0]),
				// data[1] contains 'T' for "true"
				// track over ground, magnetic
				(data[2].equals("")) ? 0.0 : Double.parseDouble(data[2]),
				// data[3] contains 'M' for "magnetic" 
				// speed in knots
				Double.parseDouble(data[4]), 
				// data[5] contains 'N' for "knots"
				// speed in km/h
				Double.parseDouble(data[6]));
				// data[7] contains 'K' for "km/h"
		} catch (Exception exception) {
			throw new ParseException("Invalid argument", -1);		
		}
	}
	
	public String getDescription() {	
		return DESCRIPTION;
	}
	
	public int getType() {
		return Nmea0183Sentence.GPVTG_TYPE;
	}
	
	/**
	 * Returns the track over ground (true).
	 * 
	 * @return trackOverGroundTrue
	 */
	public double getTrackOverGround() {
		return this.trackOverGroundTrue;
	}
	
	/**
	 * Returns the track over ground (magnetic).
	 * 
	 * @return trackOverGroundMagnetic
	 */
	public double getTrackOverGroundMagnetic() {
		return this.trackOverGroundMagnetic;
	}
	
	/**
	 * Returns the speed over ground in knots.
	 * 
	 * @return speedInKnots
	 */
	public double getSpeedInKnots() {
		return this.speedInKnots;
	}
	
	/**
	 * Returns the speed over ground in km/h.
	 * 
	 * @return speedInKmh
	 */
	public double getSpeedInKmh() {
		return this.speedInKmh;
	}
	
	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.trackOverGroundTrue);
		msg.append(',');
		msg.append('T');
		msg.append(',');
		msg.append(this.trackOverGroundMagnetic);
		msg.append(',');
		msg.append('M');
		msg.append(',');
		msg.append(this.speedInKnots);
		msg.append(',');
		msg.append('N');
		msg.append(',');
		msg.append(this.speedInKmh);
		msg.append(',');
		msg.append('K');
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}
	
	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Description: ");
		sb.append(DESCRIPTION);
		sb.append("\n");
		sb.append("Track over ground (true): ");
		sb.append(this.trackOverGroundTrue);
		sb.append("\n");
		sb.append("Track over ground (magnetic): ");
		sb.append(this.trackOverGroundMagnetic);
		sb.append("\n");
		sb.append("Speed in knots: ");
		sb.append(this.speedInKnots);
		sb.append("\n");
		sb.append("Speed in km/h: ");
		sb.append(this.speedInKmh);
		sb.append("\n");
		
		return sb.toString();
	}
}
