/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.util.Iterator;
import java.util.Vector;

/**
 * Handles input to and output from a NMEA-0183 compatible source.
 * 
 * @author king
 * @author faerber
 */

public abstract class InputOutputHandler extends org.pi4.loclib.InputOutputHandler implements Runnable {
	protected BufferedReader reader; // the reader for parsing
	private volatile boolean continueParsing = true; // break condition to end parsing
	private Vector gprmcListeners;	// list of GprmcListener type listeners
	private Vector gpggaListeners;	// list of GpggaListener type listeners
	private Vector gpgllListeners;	// list of GpgllListener type listeners
	private Vector gpgsaListeners;	// list of GpgsaListener type listeners
	private Vector gpgsvListeners;	// list of GpgsvListener type listeners
	private Vector gpvtgListeners;	// list of GpvtgListener type listeners
	private Vector stateChangeListeners; // list of state change listeners
	
	/**
	 * Returns a Singleton instance of <code>SerialPortInputOutputHandler</code>.
	 * There can only be one single instance of an InputOutputHandler for a specific port.
	 * @param comPort	the COM port to be used
	 * @return	an InputOutputHandler for the specified serial port
	 * @throws IOException
	 */
	public static synchronized InputOutputHandler getInstance(int comPort) throws IOException {
		return SerialPortInputOutputHandler.getInstance(comPort);
	}
	
	/**
	 * Returns a Singleton instance of <code>BluetoothInputOutputHandler</code>.
	 * There can only be one single instance of an InputOutputHandler for a specific address and channel.
	 * @param address	the Bluetooth device address
	 * @param channel	the RFCOMM channel
	 * @return	an InputOutputHandler for the specified address and channel
	 * @throws IOException
	 */
	public static synchronized InputOutputHandler getInstance(String address, int channel) throws IOException {
		return BluetoothInputOutputHandler.getInstance(address, channel);
	}
	
	/**
	 * Returns a Singleton instance of <code>FileInputOutputHandler</code> or <code>SerialFileInputOutputHandler</code>.
	 * There can only be one single instance of an InputOutputHandler for a specific file.
	 * 
	 * @param file	the file from which NMEA-0183 sentences (and time deltas if <code>recorded</code> is <code>true</code>) are read
	 * 
	 * @return	an InputOutputHandler for the specified file
	 * 
	 * @throws FileNotFoundException
	 */
	public static synchronized InputOutputHandler getInstance(File file, boolean recorded) throws FileNotFoundException {
		if (recorded)
			return FileInputOutputHandler.getInstance(file);
		return SerialFileInputOutputHandler.getInstance(file);
	}
	
	
	
	/**
	 * Creates a new InputOutputHandler for NMEA-0183 parsing.
	 * Attempts to load the gps.properties file.
	 */
	protected InputOutputHandler() {
		gprmcListeners				= new Vector();
		gpggaListeners				= new Vector();
		gpgllListeners					= new Vector();
		gpgsaListeners				= new Vector();
		gpgsvListeners				= new Vector();
		gpvtgListeners				= new Vector();
		stateChangeListeners 	= new Vector();
	}
	
	/**
	 * Parses the NMEA-0183 stream.
	 * This method shall invoke the <code>parse</code> method.
	 */
	abstract public void run();
	
	/**
	 * Initializes the InputOutputHandler
	 * 
	 * @param reader the reader
	 */
	protected synchronized void init(BufferedReader reader) {
		this.reader = reader;
		start();
	}
	
	/**
	 * Starts parsing the NMEA-0183 input stream.
	 * 
	 * @throws IOException
	 */
	protected void parse() throws IOException {
		while (continueParsing) {
			try {
				String line = readMessage();
				Nmea0183Sentence sentence = Nmea0183Parser.parse(line);
				if (sentence.getType() == Nmea0183Sentence.GPRMC_TYPE) {
					synchronized (gprmcListeners) {
						Iterator it = gprmcListeners.iterator();
						while (it.hasNext()) {
							GprmcListener listener = (GprmcListener) it.next();
							listener.handleGprmc((Gprmc) sentence);
						}
					}
				} else
					if (sentence.getType() == Nmea0183Sentence.GPGGA_TYPE) {
						synchronized (gpggaListeners) {
							Iterator it = gpggaListeners.iterator();
							while (it.hasNext()) {
								GpggaListener listener = (GpggaListener) it.next();
								listener.handleGpgga((Gpgga) sentence);
							}
						}
				} else
					if (sentence.getType() == Nmea0183Sentence.GPGLL_TYPE) {
						synchronized (gpgllListeners) {
							Iterator it = gpgllListeners.iterator();
							while (it.hasNext()) {
								GpgllListener listener = (GpgllListener) it.next();
								listener.handleGpgll((Gpgll) sentence);
							}
						}
				} else
					if (sentence.getType() == Nmea0183Sentence.GPGSA_TYPE) {
						synchronized (gpgsaListeners) {
							Iterator it = gpgsaListeners.iterator();
							while (it.hasNext()) {
								GpgsaListener listener = (GpgsaListener) it.next();
								listener.handleGpgsa((Gpgsa) sentence);
							}
						}
				} else
					if (sentence.getType() == Nmea0183Sentence.GPGSV_TYPE) {
						synchronized (gpgsvListeners) {
							Iterator it = gpgsvListeners.iterator();
							while (it.hasNext()) {
								GpgsvListener listener = (GpgsvListener) it.next();
								listener.handleGpgsv((Gpgsv) sentence);
							}
						}
				} else
					if (sentence.getType() == Nmea0183Sentence.GPVTG_TYPE) {
						synchronized (gpvtgListeners) {
							Iterator it = gpvtgListeners.iterator();
							while (it.hasNext()) {
								GpvtgListener listener = (GpvtgListener) it.next();
								listener.handleGpvtg((Gpvtg) sentence);
							}
						}
					}
			} catch (IOException ioe) {
				System.err.println(ioe.getLocalizedMessage());
				close();
				throw ioe;
			} catch (ParseException ex) {
				System.err.println(ex.getLocalizedMessage());
			}
		}
	}
	
	/**
	 * Reads an NMEA-0183 message (usually a line)
	 * 
	 * @return	the message without trailing CRLF
	 * 
	 * @throws IOException
	 * @throws ParseException
	 */
	protected String readMessage() throws IOException, ParseException {
		synchronized (reader) {
			return reader.readLine();
		}
	}
	
	/**
	 * Adds a GprmcListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addGprmcListener(GprmcListener listener) {
		synchronized (gprmcListeners) {
			gprmcListeners.add(listener);
		}
	}
	
	/**
	 * Removes a GprmcListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeGprmcListener(GprmcListener listener) {
		synchronized (gprmcListeners) {
			gprmcListeners.remove(listener);
		}
	}
	
	/**
	 * Adds a GpggaListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addGpggaListener(GpggaListener listener) {
		synchronized (gpggaListeners) {
			gpggaListeners.add(listener);
		}
	}
	
	/**
	 * Removes a GpggaListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeGpggaListener(GpggaListener listener) {
		synchronized (gpggaListeners) {
			gpggaListeners.remove(listener);
		}
	}
	
	/**
	 * Adds a GpgllListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addGpgllListener(GpgllListener listener) {
		synchronized (gpgllListeners) {
			gpgllListeners.add(listener);
		}
	}
	
	/**
	 * Removes a GpgllListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeGpgllListener(GpgllListener listener) {
		synchronized (gpgllListeners) {
			gpgllListeners.remove(listener);
		}
	}
	
	/**
	 * Adds a GpgsaListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addGpgsaListener(GpgsaListener listener) {
		synchronized (gpgsaListeners) {
			gpgsaListeners.add(listener);
		}
	}
	
	/**
	 * Removes a GpgsaListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeGpgsaListener(GpgsaListener listener) {
		synchronized (gpgsaListeners) {
			gpgsaListeners.remove(listener);
		}
	}
	
	/**
	 * Adds a GpgsvListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addGpgsvListener(GpgsvListener listener) {
		synchronized (gpgsvListeners) {
			gpgsvListeners.add(listener);
		}
	}
	
	/**
	 * Removes a GpgsvListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeGpgsvListener(GpgsvListener listener) {
		synchronized (gpgsvListeners) {
			gpgsvListeners.remove(listener);
		}
	}
	
	/**
	 * Adds a GpvtgListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addGpvtgListener(GpvtgListener listener) {
		synchronized (gpvtgListeners) {
			gpvtgListeners.add(listener);
		}
	}
	
	/**
	 * Removes a GpvtgListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeGpvtgListener(GpvtgListener listener) {
		synchronized (gpvtgListeners) {
			gpvtgListeners.remove(listener);
		}
	}
	
	/**
	 * Adds a StateChangeListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void addStateChangeListener(StateChangeListener listener) {
		synchronized (stateChangeListeners) {
			stateChangeListeners.add(listener);
		}
	}
	
	/**
	 * Removes a StateChangeListener type listener.
	 * 
	 * @param listener	the listener
	 */
	public void removeStateChangeListener(StateChangeListener listener) {
		synchronized (stateChangeListeners) {
			stateChangeListeners.remove(listener);
		}
	}
	
	/**
	 * Ends parsing and closes all resources.
	 */
	public void close() {
		try {
			continueParsing = false;
			super.close();			
			reader.close();
			notifyStateChangeListener();
		} catch (IOException ex) {
		}
	}
	
	/**
	 * Notifies listeners about the state change.
	 */
	protected void notifyStateChangeListener() {
		// iterate through the state change listener vector to inform the listeners about the state change
		synchronized (stateChangeListeners) {
			Iterator it = stateChangeListeners.iterator();
			while (it.hasNext()) {
				StateChangeListener listener = (StateChangeListener) it.next();
				listener.stateChanged(StateChangeListener.OUT_OF_SERVICE);
			}
		}
	}
}
