/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * Parser for NMEA-0183 sentences
 * 
 * @author faerber
 */
public class Nmea0183Parser {
	
	private Nmea0183Parser() {}
	
	/**
	 * Parses the NMEA-0183 sentence.
	 * 
	 * @param sentence	the NMEA-0183 sentence to be parsed
	 * 
	 * @return	a <code>NMEA0183Sentence</code> object
	 * 
	 * @throws ParseException
	 * @throws UnsupportedEncodingException
	 */
	public static Nmea0183Sentence parse(String sentence) throws ParseException, UnsupportedEncodingException {
		if (!check(sentence))
			throw new ParseException("Invalid NMEA-0183 sentence.", 0);
		//System.err.println(sentence);
		int beginChecksum = sentence.lastIndexOf('*'); // index of *
		String message; // NMEA-0183 message without '$' and checksum
		if (beginChecksum == sentence.length()-3) {
			message = sentence.substring(1, beginChecksum);
		} else message = sentence.substring(1);
		StringTokenizer tok = new StringTokenizer(message, ",", true); // tokenizer for sentence
		Vector components = new Vector(); // message name and arguments of the message
		while (tok.hasMoreTokens()) {
			String token = tok.nextToken();
			if (!token.equals(",")) {
				components.add(token);
				if (tok.hasMoreTokens()) {
					tok.nextToken(); // a comma
					if (!tok.hasMoreTokens())
						components.add(""); // last component is empty
				}
			} else {
				components.add(""); // previous component was empty
				if (!tok.hasMoreTokens())
					components.add(""); // last component is empty
			}
		}
		return Nmea0183Sentence.parse(components);
	}
	
	/**
	 * Checks whether the sentence is valid according to its checksum and its beginning.
	 * 
	 * @param sentence	the NMEA-0183 sentence to be checked
	 * 
	 * @return	true if valid, false otherwise
	 * 
	 * @throws ParseException
	 * @throws UnsupportedEncodingException
	 */
	private static boolean check(String sentence) throws ParseException, UnsupportedEncodingException {
		if (sentence == null)
			return false;
		if (sentence.length() <= 3) // $ + P + one
			return false;
		if (sentence.charAt(0) != '$')
			return false;
		if (sentence.charAt(1) != 'P') { // standard five-char message ID
			if (sentence.length() <= 1+5)
				return false;
			for (int i = 1; i < 1+5; i++) {
				if (!Character.isLetter(sentence.charAt(i)))
					return false;
			}
		}
		int beginChecksum = sentence.lastIndexOf('*'); // index of '*'
		if (beginChecksum != sentence.length()-3)
			return true;
		// everything between $ and * ...
		String checksum = calculateChecksum(sentence.substring(1, beginChecksum));
		return sentence.substring(beginChecksum+1).compareToIgnoreCase(checksum) == 0;
	}
	
	/**
	 * Calculates the checksum of a NMEA-0183 message
	 * 
	 * @param sentence	the NMEA-0183 sentence without the checksum and without '$'
	 * 
	 * @return	the calculated checksum
	 * 
	 * @throws ParseException
	 * @throws UnsupportedEncodingException
	 */
	public static String calculateChecksum(String sentence) throws ParseException, UnsupportedEncodingException {
		if ((sentence == null) || (sentence.length() == 0))
			throw new ParseException("Invalid argument: "+sentence, 0);
		byte[] bytes = sentence.getBytes("US-ASCII"); // encoded bytes of the sentence
		if (bytes.length == 0)
			throw new ParseException("Unexpected byte length.", 0);
		byte check = bytes[0];
		for (int i = 1; i < bytes.length; i++) {
			check ^= bytes[i];
		}
		String checksum = Integer.toHexString((int) (check & 0xff));
		if (checksum.length() == 1)
			checksum = "0"+checksum;
		return checksum;
	}
}
