/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;
import java.util.Vector;

/**
 * A NMEA-0183 sentence.
 * 
 * @author faerber
 */
public abstract class Nmea0183Sentence {
	
	/**
	 * Parses a NMEA-0183 sentence.
	 * 
	 * @param sentence	the individual components of the sentence, including the message name, not including the checksum
	 * 
	 * @return	an NMEA-0183 sentence object
	 */
	public static Nmea0183Sentence parse(Vector sentence) throws ParseException {
		String name = (String) sentence.get(0);
		sentence.remove(0);
		String[] components = new String[sentence.size()];
		sentence.copyInto(components);
		if (name.equals("GPRMC")) {
			return Gprmc.parse(components);
		} else if (name.equals("GPGGA")) {
			return Gpgga.parse(components);
		} else if (name.equals("GPGLL")) {
			return Gpgll.parse(components);
		} else if (name.equals("GPGSA")) {
			return Gpgsa.parse(components);
		} else if (name.equals("GPGSV")) {
			return Gpgsv.parse(components);
		} else if (name.equals("GPVTG")) {
			return Gpvtg.parse(components);
		} else throw new ParseException("NMEA-0183 message not recognized.", 1);
	}
	
	
	/** GGA message for GPS */
	public static final int GPGGA_TYPE = 0;
	
	/** GLL message for GPS */
	public static final int GPGLL_TYPE = 1;
	
	/** GSA message for GPS */
	public static final int GPGSA_TYPE = 2;
		
	/** GSV message for GPS */
	public static final int GPGSV_TYPE = 3;
	
	/** RMC message for GPS */
	public static final int GPRMC_TYPE = 4;
	
	/** VTG message for GPS */
	public static final int GPVTG_TYPE = 5;
	
	/** Message 100 for Sirf based GPS devices */
	public static final int PSRF100_TYPE = 6;
	
	/** Message 103 for Sirf based GPS devices */
	public static final int PSRF103_TYPE = 7;
	
	
	long time;
	
	/**
	 * Creates a new sentence object.
	 */
	protected Nmea0183Sentence() {
		time = System.currentTimeMillis();
	}
	
	/**
	 * Returns the type of this sentence object.
	 * 
	 * @return	the enumerated type
	 */
	public abstract int getType();
	
	/**
	 * Returns the name of the NMEA-0183 message.
	 * 
	 * @return	the message name
	 */
	public abstract String getDescription();
	
	/**
	 * Returns the time this object was constructed.
	 * 
	 * @return	the system time in ms
	 */
	public long getTimeConstructed() {
		return time;
	}
	
	/**
	 * Returns a String representation for debugging purposes.
	 * 
	 * @return	a verbose string listing the internal data of the message class
	 */
	public abstract String dump();
	
	/**
	 * Returns a String representation of the NMEA-0183 message.
	 * 
	 * @return	a string presenting a NMEA sentence.
	 */
	public abstract String toString();
}
