/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;

/**
 * Represents a PSRF100 proprietary NMEA-0183 message by SiRF.
 * 
 * @author faerber
 */
final public class Psrf100 extends Nmea0183Sentence {

	private static final String DESCRIPTION = "PSRF100";
	private int protocol; // the communication protocol
	private int baudrate; // the baudrate
	private int dataBits; // the data bits
	private int stopBits; // the stop bits
	private int parity; // the parity
	
	/** Sirf binary protocol */
	public static final int SIRF_PROTOCOL = 0;
	
	/** NMEA-0183 protocol */
	public static final int NMEA_PROTOCOL = 1;
	
	
	/** no parity */
	public static final int NO_PARITY = 0;
	
	/** odd parity */
	public static final int ODD_PARITY = 1;
	
	/** even parity */
	public static final int EVEN_PARITY = 2;
	
	
	/**
	 * Creates a new instance using the specified parameters.
	 * 
	 * @param protocol
	 * @param baudrate
	 * @param dataBits
	 * @param stopBits
	 * @param parity
	 * 
	 * @throws IllegalArgumentException
	 */
	public Psrf100(int protocol, int baudrate, int dataBits, int stopBits, int parity) throws IllegalArgumentException {
		setProtocol(protocol);
		setBaudrate(baudrate);
		setDataBits(dataBits);
		setStopBits(stopBits);
		setParity(parity);
	}

	public int getType() {
		return Nmea0183Sentence.PSRF100_TYPE;
	}
	
	public String getDescription() {
		return DESCRIPTION;
	}
	
	/**
	 * Sets the protocol.
	 * 
	 * @param protocol	the communication protocol (0 or 1)
	 * 
	 * @throws IllegalArgumentException	protocol is unsupported
	 */
	public void setProtocol(int protocol) throws IllegalArgumentException {
		if (!((protocol == NMEA_PROTOCOL) || (protocol == SIRF_PROTOCOL)))
			throw new IllegalArgumentException("Invalid protocol.");
		this.protocol = protocol;
	}
	
	/**
	 * Sets the baudrate.
	 * 
	 * @param baudrate	4800, 9600, 19200 or 38400
	 * 
	 * @throws IllegalArgumentException	protocol is unsupported
	 */
	public void setBaudrate(int baudrate) throws IllegalArgumentException {
		if (!((baudrate == 4800) || (baudrate == 9600) || (baudrate == 19200) || (baudrate == 38400)))
			throw new IllegalArgumentException("Invalid baudrate.");
		this.baudrate = baudrate;
	}
	
	/**
	 * Sets the number of data bits.
	 * 
	 * @param dataBits	8 or 7
	 * 
	 * @throws IllegalArgumentException	data bits are unsupported
	 */
	public void setDataBits(int dataBits) throws IllegalArgumentException {
		if (!((dataBits == 8) || (dataBits == 7)))
			throw new IllegalArgumentException("Invalid number of data bits.");
		this.dataBits = dataBits;
	}
	
	/**
	 * Sets the number of stop bits.
	 * 
	 * @param stopBits	0 or 1
	 * 
	 * @throws IllegalArgumentException	stop bits are unsupported
	 */
	public void setStopBits(int stopBits) throws IllegalArgumentException {
		if (!((stopBits == 0) || (stopBits == 1)))
			throw new IllegalArgumentException("Invalid number of stop bits.");
		this.stopBits = stopBits;
	}
	
	/**
	 * Sets the type of parity.
	 * 
	 * @param parity
	 * 
	 * @throws IllegalArgumentException	parity is unsupported
	 */
	public void setParity(int parity) throws IllegalArgumentException {
		if (!((parity == NO_PARITY) || (parity == ODD_PARITY) || (parity == EVEN_PARITY)))
			throw new IllegalArgumentException("Invalid type of parity.");
		this.parity = parity;
	}
	
	public static Psrf100 parse(String[] data) throws ParseException {
		try {
			return new Psrf100(
				// protocol
				Integer.parseInt(data[0]),
				// baudrate
				Integer.parseInt(data[1]),
				// data bits
				Integer.parseInt(data[2]),
				// stop bits
				Integer.parseInt(data[3]),
				// parity
				Integer.parseInt(data[4])
			);
		} catch (Exception ex) {
			throw new ParseException("Invalid PSRF100 message.", -1);
		}
	}

	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Protocol: ");
		sb.append(this.protocol);
		sb.append("\nBaudrate: ");
		sb.append(this.baudrate);
		sb.append("\nData bits: ");
		sb.append(this.dataBits);
		sb.append("\nStop bits: ");
		sb.append(this.stopBits);
		sb.append("\nParity: ");
		sb.append(this.parity);
		return sb.toString();
	}

	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.protocol);
		msg.append(',');
		msg.append(this.baudrate);
		msg.append(',');
		msg.append(this.dataBits);
		msg.append(',');
		msg.append(this.stopBits);
		msg.append(',');
		msg.append(this.parity);
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}
}
