/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.text.ParseException;

/**
 * A PSRF103 proprietary NMEA-0183 message by Sirf.
 * 
 * @author faerber
 */
final public class Psrf103 extends Nmea0183Sentence {

	static final String DESCRIPTION = "PSRF103";
	private int message; // the message number
	private int mode; // the operation mode
	private byte rate; // the rate
	private boolean enableChecksum; // should the checksum be enabled?

	/**
	 * Creates a new Psrf103 object.
	 * 
	 * @param message
	 * @param mode
	 * @param rate
	 * @param enableChecksum
	 * 
	 * @throws IllegalArgumentException
	 */
	public Psrf103(int message, int mode, byte rate, boolean enableChecksum) throws IllegalArgumentException {
		setProtocolParameters(message, mode, rate, enableChecksum);
	}

	public int getType() {
		return PSRF103_TYPE;
	}
	
	public String getDescription() {
		return DESCRIPTION;
	}
	
	/**
	 * Sets all message parameters at once.
	 * Subsequent calls substitute the previous values.
	 * 
	 * @param message
	 * @param mode
	 * @param rate
	 * @param checksum
	 * 
	 * @throws IllegalArgumentException
	 */
	public void setProtocolParameters(int message, int mode, byte rate, boolean checksum) throws IllegalArgumentException {
		if ((message < 0) || (message > 5))
			throw new IllegalArgumentException("Invalid message number.");
		if ((mode < 0) || (mode > 1))
			throw new IllegalArgumentException("Invalid mode.");
		this.message = message;
		this.mode = mode;
		this.rate = rate;
		this.enableChecksum = checksum;
	}
	
	public static Psrf103 parse(String[] data) throws ParseException {
		try {
			return new Psrf103(
				// message
				Integer.parseInt(data[0], 16),
				// mode
				Integer.parseInt(data[1]),
				// rate
				Byte.parseByte(data[2], 16),
				// enableChecksum
				data[3].equals("01")
			);
		} catch (Exception ex) {
			throw new ParseException("Invalid PSRF103 message.", -1);
		}
	}

	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("Message: ");
		sb.append(this.message);
		sb.append("\nMode: ");
		sb.append(this.mode);
		sb.append("\nRate: ");
		sb.append(this.rate);
		sb.append("\nEnable Checksum: ");
		sb.append(this.enableChecksum);
		return sb.toString();
	}

	public String toString() {
		StringBuffer msg = new StringBuffer(DESCRIPTION);
		msg.append(',');
		msg.append(this.message);
		msg.append(',');
		msg.append(this.mode);
		msg.append(',');
		msg.append(this.rate);
		msg.append(',');
		msg.append((this.enableChecksum) ? "1" : "0");
		try {
			String checksum = Nmea0183Parser.calculateChecksum(msg.toString());
			msg.insert(0, '$');
			msg.append('*');
			msg.append(checksum);
		} catch (Exception ex) {
			System.err.println(ex.getLocalizedMessage());
		}
		return msg.toString();
	}	
}
