/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

/**
 * Stores information about a satellite (ID, elevation, azimuth, signal-to-noise ratio (SNR))
 * 
 * @author abiskop
 */
public class SatelliteInfo {
	static final int SNR_NOT_VALID = -1;
	private int satelliteId;
	private int elevation;
	private int azimuth;
	private int snr = SNR_NOT_VALID;

	/**
	 * Constructor for <code>SatelliteInfo</code>.
	 * 
	 * @param satelliteId
	 * @param elevation
	 * @param azimuth
	 * @param snr Signal-to-noise ratio 
	 */
	public SatelliteInfo(int satelliteId, int elevation, int azimuth, int snr) {
		this.satelliteId = satelliteId;
		this.elevation = elevation;
		this.azimuth = azimuth;
		this.snr = snr;
	}
	
	/**
	 * Returns the satellite's ID number.
	 * 
	 * @return satelliteId
	 */
	public int getSatelliteId() {
		return this.satelliteId;
	}
	
	/**
	 * Returns the elevation of the satellite.
	 * 
	 * @return elevation
	 */
	public int getElevation() {
		return this.elevation;
	}
	
	/**
	 * Returns the azimuth of the satellite.
	 * 
	 * @return azimuth
	 */
	public int getAzimuth() {
		return this.azimuth;
	}
	
	/**
	 * Returns the signal-to-noise ratio of the satellite.
	 * 
	 * @see <a href=http://en.wikipedia.org/wiki/Signal-to-noise_ratio>http://en.wikipedia.org/wiki/Signal-to-noise_ratio</a>
	 * 
	 * @return snr Signal-to-noise ratio or -1 if Signal-to-noise ratio is not set or valid
	 */
	public int getSnr() {
		return this.snr;
	}
	
	/**
	 * Returns a detailed <code>String</code> representation of this <code>SatelliteInfo</code> object.
	 * 
	 * @return stringRepresentation
	 */
	public String dump() {
		StringBuffer sb = new StringBuffer();
		sb.append("ID: ");
		sb.append(this.satelliteId);
		sb.append("\n");
		sb.append("Elevation: ");
		sb.append(this.elevation);
		sb.append("\n");
		sb.append("Azimuth: ");
		sb.append(this.azimuth);
		sb.append("\n");
		sb.append("SNR: ");
		if (this.snr != SNR_NOT_VALID) {
			sb.append(this.snr);
		} else {
			sb.append("n/a");
		}
		sb.append("\n");
		
		return sb.toString();
	}
	
	/**
	 * Returns a NMEA sentence compliant <code>String</code> representation of this <code>SatelliteInfo</code> object.
	 * 
	 * @return stringRepresentation
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer(Integer.toString(this.satelliteId));
		buffer.append(',');
		buffer.append(this.elevation);
		buffer.append(',');
		buffer.append(this.azimuth);
		buffer.append(',');
		if (this.snr != SNR_NOT_VALID) buffer.append(this.snr);
		return buffer.toString();		
	}
}
