/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.io.IOException;
import java.text.ParseException;
import java.util.*;
import java.io.*;

/**
 * This class contains a InputOutputHandler for serial port communication through a /dev/XX file.
 * 
 * @author king
 * 
 */
public class SerialFileInputOutputHandler extends InputOutputHandler
	implements org.pi4.loclib.FileInputOutputHandler {
	
	private final File device; // the device
	private static HashMap instances = null; // the Singleton instances
	
	public synchronized static InputOutputHandler getInstance(File device) throws FileNotFoundException {
		if (instances == null) {
			instances = new HashMap();
		}
		if (instances.containsKey(device)) {
			return (InputOutputHandler) instances.get(device);
		} else {
			SerialFileInputOutputHandler sfioh = new SerialFileInputOutputHandler(device);
			instances.put(device, sfioh);
			return sfioh;
		}
	}
	
	/**
	 * Creates a new InputOutputHandler for the specified file.
	 * 
	 * @param deviceName	the device name
	 * 
	 * @throws FileNotFoundException
	 */
	protected SerialFileInputOutputHandler(File device) throws FileNotFoundException {
		this.device = device;
		init(new BufferedReader(new InputStreamReader(new FileInputStream(device))));
	}
	
	public void run() {
		try {
			parse();
		} catch (IOException iox) {
			synchronized (instances) {
				instances.remove(device);
			}
		}
	}
	
	/**
	 * Returns the dev file from which it reads.
	 * 
	 * @return the device
	 */
	public File getFile() {
		return device;
	}
	
	protected synchronized String readMessage() throws IOException, ParseException {
		// This is a bugfix for Familiar Linux (iPaq) where an empty line is retrieved between messages
		while (true) {
			String line = super.readMessage();
			if (line == null)
				throw new ParseException("Possibly zero size file?", -1);
			if (line.length() > 0)	return line;
		}
	}
	
	public void close() {		
		super.close();
		synchronized (instances) {
			instances.remove(device);
		}
	}
}
