/*
 * This file is part of Loclib.
 *
 * Loclib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Loclib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Loclib; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package org.pi4.nmea0183;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;

import org.pi4.loclib.Configuration;

/**
 * A serial port based InputOutputHandler.
 * Expects a NMEA-0183 compatible GPS receiver.
 * 
 * @author faerber
 */
public class SerialPortInputOutputHandler extends InputOutputHandler implements org.pi4.loclib.SerialPortInputOutputHandler {
	private final int port; // the COM port
	private StreamConnection conn; // the serial connection
	private static HashMap instances = null; // Singleton instances mapped to their port numbers
	
	public synchronized static InputOutputHandler getInstance(int com) throws IOException {
		if (instances == null) {
			instances = new HashMap();
		}
		Integer i = new Integer(com);
		if (instances.containsKey(i)) {
			return (InputOutputHandler) instances.get(i);
		} else {
			SerialPortInputOutputHandler spioh = new SerialPortInputOutputHandler(com);
			instances.put(i, spioh);
			return spioh;
		}
	}
	
	/**
	 * Creates a new serial port based InputOutputHandler for a specific COM port.
	 * 
	 * @param port	the COM port
	 * 
	 * @throws IOException
	 */
	protected SerialPortInputOutputHandler(int port) throws IOException {
		this.port = port;
		try {
			boolean bSirf = Configuration.getProperty("GPS.SiRF", "on").equalsIgnoreCase("on");
			if (bSirf) { // should SiRF-specific messages be used?
				// send communication setup messages
				StringBuffer uri = new StringBuffer("comm:");
				uri.append(port);
				uri.append(";baudrate=4800;bitsperchar=8;stopbits=1;parity=none;autocts=");
				uri.append(Configuration.getProperty("GPS.serial.autocts", "off"));
				uri.append(";autorts=");
				uri.append(Configuration.getProperty("GPS.serial.autorts", "off"));
				uri.append(";blocking=");
				uri.append(Configuration.getProperty("GPS.serial.blocking", "off"));
				conn = (StreamConnection) Connector.open(uri.toString());
				BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(conn.openOutputStream(), "US-ASCII"));
				String strParity = Configuration.getProperty("GPS.serial.parity", "none");
				int parity = Psrf100.NO_PARITY;
				if (strParity.equalsIgnoreCase("odd")) {
					parity = Psrf100.ODD_PARITY;
				} else if (strParity.equalsIgnoreCase("even")) {
					parity = Psrf100.EVEN_PARITY;
				}
				Psrf100 psrf100 = new Psrf100(Psrf100.NMEA_PROTOCOL,
					Integer.parseInt(Configuration.getProperty("GPS.serial.baudrate", "4800")),
					Integer.parseInt(Configuration.getProperty("GPS.serial.bitsperchar", "8")),
					Integer.parseInt(Configuration.getProperty("GPS.serial.stopbits", "1")),
					parity
				);
				writer.write(psrf100.toString());
				writer.write("\r\n");
				writer.flush();
				writer.close();
				conn.close();
			}
		
			// construction of Connector URI
			StringBuffer uri = new StringBuffer("comm:");
			uri.append(port);
			uri.append(";baudrate=");
			uri.append(Configuration.getProperty("GPS.serial.baudrate", "4800"));
			uri.append(";bitsperchar=");
			uri.append(Configuration.getProperty("GPS.serial.bitsperchar", "8"));
			uri.append(";stopbits=");
			uri.append(Configuration.getProperty("GPS.serial.stopbits", "1"));
			uri.append(";parity=");
			uri.append(Configuration.getProperty("GPS.serial.parity", "none"));
			uri.append(";autocts=");
			uri.append(Configuration.getProperty("GPS.serial.autocts", "off"));
			uri.append(";autorts=");
			uri.append(Configuration.getProperty("GPS.serial.autorts", "off"));
			uri.append(";blocking=");
			uri.append(Configuration.getProperty("GPS.serial.blocking", "off"));
		
			conn = (StreamConnection) Connector.open(uri.toString());
		
			if (bSirf) {
				// switch on/off specific nmea sentences
				BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(conn.openOutputStream(), "US-ASCII"));
				Psrf103 psrf103 = new Psrf103(Nmea0183Sentence.GPRMC_TYPE, 0,
					Byte.parseByte(Configuration.getProperty("GPS.GPRMC.rate", "1")),
					Configuration.getProperty("GPS.GPRMC.checksum", "on").equals("on"));
				writer.write(psrf103.toString());
				writer.write("\r\n");
				psrf103.setProtocolParameters(Nmea0183Sentence.GPGGA_TYPE, 0,
					Byte.parseByte(Configuration.getProperty("GPS.GPGGA.rate", "1")),
					Configuration.getProperty("GPGGA.checksum", "on").equals("on"));
				writer.write(psrf103.toString());
				writer.write("\r\n");
				psrf103.setProtocolParameters(Nmea0183Sentence.GPGLL_TYPE, 0,
					Byte.parseByte(Configuration.getProperty("GPGLL.rate", "1")),
					Configuration.getProperty("GPS.GPGLL.checksum", "on").equals("on"));
				writer.write(psrf103.toString());
				writer.write("\r\n");
				psrf103.setProtocolParameters(Nmea0183Sentence.GPGSA_TYPE, 0,
					Byte.parseByte(Configuration.getProperty("GPS.GPGSA.rate", "1")),
					Configuration.getProperty("GPS.GPGSA.checksum", "on").equals("on"));
				writer.write(psrf103.toString());
				writer.write("\r\n");
				psrf103.setProtocolParameters(Nmea0183Sentence.GPGSV_TYPE, 0,
					Byte.parseByte(Configuration.getProperty("GPS.GPGSV.rate", "1")),
					Configuration.getProperty("GPGSV.checksum", "on").equals("on"));
				writer.write(psrf103.toString());
				writer.write("\r\n");
				psrf103.setProtocolParameters(Nmea0183Sentence.GPVTG_TYPE, 0,
					Byte.parseByte(Configuration.getProperty("GPS.GPVTG.rate", "1")),
					Configuration.getProperty("GPS.GPVTG.checksum", "on").equals("on"));
				writer.write(psrf103.toString());
				writer.write("\r\n");
				writer.flush();
				writer.close();
			}
				
			// init reader and start parsing
			init(new BufferedReader(new InputStreamReader(conn.openInputStream(), "US-ASCII")));
		} catch (IllegalArgumentException ex) {
			System.err.println(ex.getLocalizedMessage());
		}
	}
	
	public void run() {
		try {
			parse();
		} catch (IOException iox) {
			synchronized (instances) {
				instances.remove(new Integer(port));
			}
		}
	}
	
	
	/**
	 * Gets the COM port number.
	 * 
	 * @return	port number
	 */
	public int getPort() {
		return port;
	}
	
	public void close() {
		try {
			super.close();
			conn.close();
		} catch (IOException ex) {
		} finally {
			synchronized (instances) {
				instances.remove(new Integer(port));
			}
		}
	}
}
